@@ -1,4 +1,4 @@
-<?php // $Id: event.php,v 1.22.4.1 2004/08/25 05:03:45 moodler Exp $
+<?php // $Id: event.php,v 1.74.2.7 2009/11/19 19:26:38 skodak Exp $
 
 /////////////////////////////////////////////////////////////////////////////
 //                                                                         //
@@ -9,14 +9,14 @@
 // Copyright (C) 2003-2004  Greek School Network            www.sch.gr     //
 //                                                                         //
 // Designed by:                                                            //
-//     Avgoustos Tsinakos (tsinakos@uom.gr)                                //
-//     Jon Papaioannou (pj@uom.gr)                                         //
+//     Avgoustos Tsinakos (tsinakos@teikav.edu.gr)                         //
+//     Jon Papaioannou (pj@moodle.org)                                     //
 //                                                                         //
 // Programming and development:                                            //
-//     Jon Papaioannou (pj@uom.gr)                                         //
+//     Jon Papaioannou (pj@moodle.org)                                     //
 //                                                                         //
 // For bugs, suggestions, etc contact:                                     //
-//     Jon Papaioannou (pj@uom.gr)                                         //
+//     Jon Papaioannou (pj@moodle.org)                                     //
 //                                                                         //
 // The current module was developed at the University of Macedonia         //
 // (www.uom.gr) under the funding of the Greek School Network (www.sch.gr) //
@@ -45,22 +45,37 @@
 
     require_login();
 
+    $action = required_param('action', PARAM_ALPHA);
+    $eventid = optional_param('id', 0, PARAM_INT);
+    $eventtype = optional_param('type', 'select', PARAM_ALPHA);
+    $urlcourse = optional_param('course', 0, PARAM_INT);
+    $cal_y = optional_param('cal_y');
+    $cal_m = optional_param('cal_m');
+    $cal_d = optional_param('cal_d');
+
     if(isguest()) {
         // Guests cannot do anything with events
-        redirect(CALENDAR_URL.'view.php?view=upcoming');
+        redirect(CALENDAR_URL.'view.php?view=upcoming&amp;course='.$urlcourse);
     }
 
-    require_variable($_REQUEST['action']);
-    optional_variable($_REQUEST['id']);
-    optional_variable($_REQUEST['type'], 'select');
-    $_REQUEST['id'] = intval($_REQUEST['id']); // Always a good idea, against SQL injections
+    $focus = '';
 
     if(!$site = get_site()) {
         redirect($CFG->wwwroot.'/'.$CFG->admin.'/index.php');
     }
 
+    $strcalendar = get_string('calendar', 'calendar');
+
+    // Initialize the session variables
+    calendar_session_vars();
+
     $now = usergetdate(time());
-    $nav = calendar_get_link_tag(get_string('calendar', 'calendar'), CALENDAR_URL.'view.php?view=upcoming&amp;', $now['mday'], $now['mon'], $now['year']);
+    $navlinks = array();
+    $calendar_navlink = array('name' => $strcalendar,
+                          'link' =>calendar_get_link_href(CALENDAR_URL.'view.php?view=upcoming&amp;course='.$urlcourse.'&amp;',
+                                                          $now['mday'], $now['mon'], $now['year']),
+                          'type' => 'misc');
+
     $day = intval($now['mday']);
     $mon = intval($now['mon']);
     $yr = intval($now['year']);
@@ -72,25 +87,27 @@
     }
 
     // If a course has been supplied in the URL, change the filters to show that one
-    if(!empty($_GET['course'])) {
-        if(is_numeric($_GET['course']) && $_GET['course'] > 0 && record_exists('course', 'id', $_GET['course'])) {
-            if($_GET['course'] == 1) {
-                // If coming from the home page, show all courses
-                $SESSION->cal_courses_shown = calendar_get_default_courses(true);
-                calendar_set_referring_course(0);
-            }
-            else {
-                // Otherwise show just this one
-                $SESSION->cal_courses_shown = intval($_GET['course']);
-                calendar_set_referring_course($SESSION->cal_courses_shown);
-            }
+    if($urlcourse > 0 && record_exists('course', 'id', $urlcourse)) {
+        require_login($urlcourse, false);
+
+        if($urlcourse == SITEID) {
+            // If coming from the site page, show all courses
+            $SESSION->cal_courses_shown = calendar_get_default_courses(true);
+            calendar_set_referring_course(0);
+        }
+        else {
+            // Otherwise show just this one
+            $SESSION->cal_courses_shown = $urlcourse;
+            calendar_set_referring_course($SESSION->cal_courses_shown);
         }
     }
 
-    switch($_REQUEST['action']) {
+    $form = null;
+
+    switch($action) {
         case 'delete':
             $title = get_string('deleteevent', 'calendar');
-            $event = get_record('event', 'id', $_REQUEST['id']);
+            $event = get_record('event', 'id', $eventid);
             if($event === false) {
                 error('Invalid event');
             }
@@ -101,7 +118,9 @@
 
         case 'edit':
             $title = get_string('editevent', 'calendar');
-            $event = get_record('event', 'id', $_REQUEST['id']);
+            $event = get_record('event', 'id', $eventid);
+            $repeats = optional_param('repeats', 0, PARAM_INT);
+
             if($event === false) {
                 error('Invalid event');
             }
@@ -109,10 +128,9 @@
                 error('You are not authorized to do this');
             }
 
-            if($form = data_submitted()) {
+            if($form = data_submitted() and confirm_sesskey()) {
 
-                $form->name = strip_tags($form->name);  // Strip all tags
-                //$form->description = clean_text($form->description , $form->format);   // Clean up any bad tags
+                $form->name = clean_param(strip_tags($form->name,'<lang><span>'), PARAM_CLEAN);
 
                 $form->timestart = make_timestamp($form->startyr, $form->startmon, $form->startday, $form->starthr, $form->startmin);
                 if($form->duration == 1) {
@@ -122,21 +140,47 @@
                     }
                 }
                 else if($form->duration == 2) {
-                    $form->timeduration = $form->minutes * 60;
+                    $form->timeduration = $form->minutes * MINSECS;
                 }
                 else {
                     $form->timeduration = 0;
                 }
-                validate_form($form, $err);
-                if (count($err) == 0) {
-                    $form->timemodified = time();
-                    update_record('event', $form);
 
-                    /// Log the event update.
-                    add_to_log($form->courseid, 'calendar', 'edit', 'event.php?action=edit&amp;id='.$form->id, $form->name);
+                validate_form($form, $err);
+
+                if (count($err) == 0) {
+
+                    if($event->repeatid && $repeats) {
+                        // Update all
+                        if($form->timestart >= $event->timestart) {
+                            $timestartoffset = 'timestart + '.($form->timestart - $event->timestart);
+                        }
+                        else {
+                            $timestartoffset = 'timestart - '.($event->timestart - $form->timestart);
+                        }
+
+                        execute_sql('UPDATE '.$CFG->prefix.'event SET '.
+                            'name = \''.$form->name.'\','.
+                            'description = \''.$form->description.'\','.
+                            'timestart = '.$timestartoffset.','.
+                            'timeduration = '.$form->timeduration.','.
+                            'timemodified = '.time().' WHERE repeatid = '.$event->repeatid);
+
+                        /// Log the event update.
+                        add_to_log($form->courseid, 'calendar', 'edit all', 'event.php?action=edit&amp;id='.$form->id, stripslashes($form->name));
+                    }
+
+                    else {
+                        // Update this
+                        $form->timemodified = time();
+                        update_record('event', $form);
+
+                        /// Log the event update.
+                        add_to_log($form->courseid, 'calendar', 'edit', 'event.php?action=edit&amp;id='.$form->id, stripslashes($form->name));
+                    }
 
                     // OK, now redirect to day view
-                    redirect(CALENDAR_URL.'view.php?view=day&cal_d='.$form->startday.'&cal_m='.$form->startmon.'&cal_y='.$form->startyr);
+                    redirect(CALENDAR_URL.'view.php?view=day&amp;course='.$urlcourse.'&cal_d='.$form->startday.'&cal_m='.$form->startmon.'&cal_y='.$form->startyr);
                 }
                 else {
                     foreach ($err as $key => $value) {
@@ -149,10 +193,9 @@
         case 'new':
             $title = get_string('newevent', 'calendar');
             $form = data_submitted();
-            if(!empty($form) && $form->type == 'defined') {
+            if(!empty($form) && !empty($form->name) && confirm_sesskey()) {
 
-                $form->name = strip_tags($form->name);  // Strip all tags
-                //$form->description = clean_text($form->description , $form->format);   // Clean up any bad tags
+                $form->name = clean_text(strip_tags($form->name, '<lang><span>'));
 
                 $form->timestart = make_timestamp($form->startyr, $form->startmon, $form->startday, $form->starthr, $form->startmin);
                 if($form->duration == 1) {
@@ -162,12 +205,12 @@
                     }
                 }
                 else if ($form->duration == 2) {
-                    $form->timeduration = $form->minutes * 60;
+                    $form->timeduration = $form->minutes * MINSECS;
                 }
                 else {
                     $form->timeduration = 0;
                 }
-                if(!calendar_add_event_allowed($form->courseid, $form->groupid, $form->userid)) {
+                if(!calendar_add_event_allowed($form)) {
                     error('You are not authorized to do this');
                 }
                 validate_form($form, $err);
@@ -176,81 +219,116 @@
 
                     /// Get the event id for the log record.
                     $eventid = insert_record('event', $form, true);
+                    
+                    /// Use the event id as the repeatid to link repeat entries together
+                    if ($form->repeat) {
+                        $form->repeatid = $form->id = $eventid;
+                        update_record('event', $form);         // update the row, to set its repeatid        	
+                    }
 
                     /// Log the event entry.
-                    add_to_log($form->courseid, 'calendar', 'add', 'event.php?action=edit&amp;id='.$eventid, $form->name);
+                    add_to_log($form->courseid, 'calendar', 'add', 'event.php?action=edit&amp;id='.$eventid, stripslashes($form->name));
 
                     if ($form->repeat) {
                         for($i = 1; $i < $form->repeats; $i++) {
-                            $form->timestart += 604800;  // add one week
+                            // What's the DST offset for the previous repeat?
+                            $dst_offset_prev = dst_offset_on($form->timestart);
+
+                            $form->timestart += WEEKSECS;
+
+                            // If the offset has changed in the meantime, update this repeat accordingly
+                            $form->timestart += $dst_offset_prev - dst_offset_on($form->timestart);
+
                             /// Get the event id for the log record.
                             $eventid = insert_record('event', $form, true);
+
                             /// Log the event entry.
-                            add_to_log($form->courseid, 'calendar', 'add', 'event.php?action=edit&amp;id='.$eventid, $form->name);
+                            add_to_log($form->courseid, 'calendar', 'add', 'event.php?action=edit&amp;id='.$eventid, stripslashes($form->name));
                         }
                     }
-
                     // OK, now redirect to day view
-                    redirect(CALENDAR_URL.'view.php?view=day&cal_d='.$form->startday.'&cal_m='.$form->startmon.'&cal_y='.$form->startyr);
+                    redirect(CALENDAR_URL.'view.php?view=day&amp;course='.$urlcourse.'&cal_d='.$form->startday.'&cal_m='.$form->startmon.'&cal_y='.$form->startyr);
                 }
                 else {
                     foreach ($err as $key => $value) {
-                        $focus = 'form'.$key;
+                        $focus = 'form.'.$key;
                     }
                 }
             }
         break;
-    }
-    if(empty($focus)) $focus = '';
-
-    // Let's see if we are supposed to provide a referring course link
-    // but NOT for the "main page" course
-    if($SESSION->cal_course_referer > 1 &&
-      ($shortname = get_field('course', 'shortname', 'id', $SESSION->cal_course_referer)) !== false) {
-        // If we know about the referring course, show a return link
-        $nav = '<a href="'.$CFG->wwwroot.'/course/view.php?id='.$SESSION->cal_course_referer.'">'.$shortname.'</a> -> '.$nav;
+        default: // no action
+            $title='';
+        break;
     }
 
-    print_header(get_string('calendar', 'calendar').': '.$title, $site->fullname, $nav.' -> '.$title,
-                 $focus, '', true, '', '<p class="logininfo">'.user_login_string($site).'</p>');
+    $form = stripslashes_recursive($form);
+
+    if (!empty($SESSION->cal_course_referer)) {
+        // TODO: This is part of the Great $course Hack in Moodle. Replace it at some point.
+        $course = get_record('course', 'id', $SESSION->cal_course_referer);
+    } else {
+        $course = $site;
+    }
+    require_login($course, false);
+
+    $navlinks[] = $calendar_navlink;
+    $navlinks[] = array('name' => $title, 'link' => null, 'type' => 'misc');
+    $navigation = build_navigation($navlinks);
+    print_header($site->shortname.': '.$strcalendar.': '.$title, $strcalendar, $navigation,
+                 'eventform.name', '', true, '', user_login_string($site));
 
     echo calendar_overlib_html();
 
-    echo '<table border="0" cellpadding="3" cellspacing="0" width="100%"><tr valign="top">';
-    echo '<td valign="top" width="100%">';
+    echo '<table id="calendar">';
+    echo '<tr><td class="maincalendar">';
 
-    switch($_REQUEST['action']) {
+    switch($action) {
         case 'delete':
-            if(!empty($_REQUEST['confirm']) && $_REQUEST['confirm'] == 1) {
+            $confirm = optional_param('confirm', 0, PARAM_INT);
+            $repeats = optional_param('repeats', 0, PARAM_INT);
+            if($confirm and confirm_sesskey()) {
                 // Kill it and redirect to day view
-                if(($event = get_record('event', 'id', $_REQUEST['id'])) !== false) {
-                    /// Log the event delete.
+                if(($event = get_record('event', 'id', $eventid)) !== false) {
 
-                    delete_records('event', 'id', $_REQUEST['id']);
-
-                    // pj - fixed the course id problem, but now we have another one:
-                    // what to do with the URL?
-                    add_to_log($event->courseid, 'calendar', 'delete', '', $event->name);
+                    if($event->repeatid && $repeats) {
+                        delete_records('event', 'repeatid', $event->repeatid);
+                        add_to_log($event->courseid, 'calendar', 'delete all', '', $event->name);
+                    }
+                    else {
+                        delete_records('event', 'id', $eventid);
+                        add_to_log($event->courseid, 'calendar', 'delete', '', $event->name);
+                    }
                 }
 
-                if(checkdate($_REQUEST['m'], $_REQUEST['d'], $_REQUEST['y'])) {
-                    // Being a bit paranoid to check this, but it doesn't hurt
-                    redirect(CALENDAR_URL.'view.php?view=day&cal_d='.$_REQUEST['d'].'&cal_m='.$_REQUEST['m'].'&cal_y='.$_REQUEST['y']);
-                }
-                else {
-                    // Redirect to now
-                    redirect(CALENDAR_URL.'view.php?view=day&cal_d='.$now['mday'].'&cal_m='.$now['mon'].'&cal_y='.$now['year']);
-                }
+                echo '</td></tr></table>';
+                redirect(CALENDAR_URL.'view.php?view=day&amp;course='.$urlcourse.'&cal_d='.$_REQUEST['d'].'&cal_m='.$_REQUEST['m'].'&cal_y='.$_REQUEST['y']);
+
             }
             else {
                 $eventtime = usergetdate($event->timestart);
                 $m = $eventtime['mon'];
                 $d = $eventtime['mday'];
                 $y = $eventtime['year'];
+
+                if($event->repeatid) {
+                    $fetch = get_record_sql('SELECT 1, COUNT(id) AS repeatcount FROM '.$CFG->prefix.'event WHERE repeatid = '.$event->repeatid);
+                    $repeatcount = $fetch->repeatcount;
+                }
+                else {
+                    $repeatcount = 0;
+                }
+
                 // Display confirmation form
-                print_side_block_start(get_string('deleteevent', 'calendar').': '.$event->name, array('class' => 'mycalendar'));
+                echo '<div class="header">'.get_string('deleteevent', 'calendar').': '.$event->name.'</div>';
+                echo '<h2>'.get_string('confirmeventdelete', 'calendar').'</h2>';
+                if($repeatcount > 1) {
+                    echo '<p>'.get_string('youcandeleteallrepeats', 'calendar', $repeatcount).'</p>';
+                }
+                echo '<div class="eventlist">';
+                $event->time = calendar_format_event_time($event, time(), '', false);
+                calendar_print_event($event);
+                echo '</div>';
                 include('event_delete.html');
-                print_side_block_end();
             }
         break;
 
@@ -263,7 +341,7 @@
                 $form->timeduration = $event->timeduration;
                 $form->id = $event->id;
                 $form->format = $defaultformat;
-                if($event->timeduration > 3600) {
+                if($event->timeduration > HOURSECS) {
                     // More than one hour, so default to normal duration mode
                     $form->duration = 1;
                     $form->minutes = '';
@@ -271,7 +349,7 @@
                 else if($event->timeduration) {
                     // Up to one hour, "minutes" mode probably is better here
                     $form->duration = 2;
-                    $form->minutes = $event->timeduration / 60;
+                    $form->minutes = $event->timeduration / MINSECS;
                 }
                 else {
                     // No duration
@@ -279,50 +357,54 @@
                     $form->minutes = '';
                 }
             }
-            if (!empty($form->courseid)) {       // Fixes bug 1488
+
+            if (!empty($form->courseid)) {
+                // TODO: This is part of the Great $course Hack in Moodle. Replace it at some point.
                 $course = get_record('course', 'id', $form->courseid);
             } else {
                 $course = $site;
             }
 
-            print_side_block_start(get_string('editevent', 'calendar'), array('class' => 'mycalendar'));
+            if($event->repeatid) {
+                $fetch = get_record_sql('SELECT 1, COUNT(id) AS repeatcount FROM '.$CFG->prefix.'event WHERE repeatid = '.$event->repeatid);
+                $repeatcount = $fetch->repeatcount;
+            }
+            else {
+                $repeatcount = 0;
+            }
+
+            echo '<div class="header">'.get_string('editevent', 'calendar').'</div>';
             include('event_edit.html');
-            print_side_block_end();
             if ($usehtmleditor) {
                 use_html_editor("description");
             }
         break;
 
         case 'new':
-            optional_variable($_GET['cal_y']);
-            optional_variable($_GET['cal_m']);
-            optional_variable($_GET['cal_d']);
-            optional_variable($form->timestart, -1);
-
-            if($_GET['cal_y'] && $_GET['cal_m'] && $_GET['cal_d'] && checkdate($_GET['cal_m'], $_GET['cal_d'], $_GET['cal_y'])) {
-                $form->timestart = make_timestamp($_GET['cal_y'], $_GET['cal_m'], $_GET['cal_d'], 0, 0, 0);
+            if($cal_y && $cal_m && $cal_d && checkdate($cal_m, $cal_d, $cal_y)) {
+                $form->timestart = make_timestamp($cal_y, $cal_m, $cal_d, 0, 0, 0);
             }
-            else if($_GET['cal_y'] && $_GET['cal_m'] && checkdate($_GET['cal_m'], 1, $_GET['cal_y'])) {
-                if($_GET['cal_y'] == $now['year'] && $_GET['cal_m'] == $now['mon']) {
-                    $form->timestart = make_timestamp($_GET['cal_y'], $_GET['cal_m'], $now['mday'], 0, 0, 0);
+            else if($cal_y && $cal_m && checkdate($cal_m, 1, $cal_y)) {
+                if($cal_y == $now['year'] && $cal_m == $now['mon']) {
+                    $form->timestart = make_timestamp($cal_y, $cal_m, $now['mday'], 0, 0, 0);
                 }
                 else {
-                    $form->timestart = make_timestamp($_GET['cal_y'], $_GET['cal_m'], 1, 0, 0, 0);
+                    $form->timestart = make_timestamp($cal_y, $cal_m, 1, 0, 0, 0);
                 }
             }
-            if($form->timestart < 0) {
+            if(!isset($form->timestart) or $form->timestart < 0) {
                 $form->timestart = time();
             }
 
             calendar_get_allowed_types($allowed);
             if(!$allowed->groups && !$allowed->courses && !$allowed->site) {
                 // Take the shortcut
-                $_REQUEST['type'] = 'user';
+                $eventtype = 'user';
             }
 
             $header = '';
 
-            switch($_REQUEST['type']) {
+            switch($eventtype) {
                 case 'user':
                     $form->name = '';
                     $form->description = '';
@@ -337,14 +419,14 @@
                     $form->repeat = 0;
                     $form->repeats = '';
                     $form->minutes = '';
+                    $form->type = 'user';
                     $header = get_string('typeuser', 'calendar');
                 break;
                 case 'group':
-                    optional_variable($_REQUEST['groupid']);
-                    $groupid = $_REQUEST['groupid'];
-                    if(!($group = get_record('groups', 'id', $groupid) )) {
+                    $groupid = optional_param('groupid', 0, PARAM_INT);
+                    if (! ($group = groups_get_group($groupid))) { //TODO:check.
                         calendar_get_allowed_types($allowed);
-                        $_REQUEST['type'] = 'select';
+                        $eventtype = 'select';
                     }
                     else {
                         $form->name = '';
@@ -360,15 +442,15 @@
                         $form->repeat = 0;
                         $form->repeats = '';
                         $form->minutes = '';
+                        $form->type = 'group';
                         $header = get_string('typegroup', 'calendar');
                     }
                 break;
                 case 'course':
-                    optional_variable($_REQUEST['courseid']);
-                    $courseid = $_REQUEST['courseid'];
+                    $courseid = optional_param('courseid', 0, PARAM_INT);
                     if(!record_exists('course', 'id', $courseid)) {
                         calendar_get_allowed_types($allowed);
-                        $_REQUEST['type'] = 'select';
+                        $eventtype = 'select';
                     }
                     else {
                         $form->name = '';
@@ -384,13 +466,14 @@
                         $form->repeat = 0;
                         $form->repeats = '';
                         $form->minutes = '';
+                        $form->type = 'course';
                         $header = get_string('typecourse', 'calendar');
                     }
                 break;
                 case 'site':
                     $form->name = '';
                     $form->description = '';
-                    $form->courseid = 1;
+                    $form->courseid = SITEID;
                     $form->groupid = 0;
                     $form->userid = $USER->id;
                     $form->modulename = '';
@@ -401,9 +484,9 @@
                     $form->repeat = 0;
                     $form->repeats = '';
                     $form->minutes = '';
+                    $form->type = 'site';
                     $header = get_string('typesite', 'calendar');
                 break;
-                case 'defined':
                 case 'select':
                 break;
                 default:
@@ -415,20 +498,23 @@
                 $header = ' ('.$header.')';
             }
 
-            print_side_block_start(get_string('newevent', 'calendar').$header, array('class' => 'mycalendar'));
-            if($_REQUEST['type'] == 'select') {
-                $defaultcourse = $SESSION->cal_course_referer;
-                if(isteacheredit($defaultcourse, $USER->id)) {
-                    $defaultgroup = 0;
+            echo '<div class="header">'.get_string('newevent', 'calendar').$header.'</div>';
+
+            if($eventtype == 'select') {
+                $courseid = optional_param('courseid', $SESSION->cal_course_referer, PARAM_INT);
+                if ($courseid == 0) { // workaround by Dan for bug #6130
+                    $courseid = SITEID;
                 }
-                else {
-                    $defaultgroup = user_group($defaultcourse, $USER->id);
+                if (!$course = get_record('course', 'id', $courseid)) {
+                    error('Incorrect course ID');
                 }
-                optional_variable($_REQUEST['groupid'], $defaultgroup->id);
-                optional_variable($_REQUEST['courseid'], $defaultcourse);
-                $groupid = $_REQUEST['groupid'];
-                $courseid = $_REQUEST['courseid'];
+                
+                $groupid = groups_get_course_group($course);
+
+                echo '<h2>'.get_string('eventkind', 'calendar').':</h2>';
+                echo '<div id="selecteventtype">';
                 include('event_select.html');
+                echo '</div>';
             }
             else {
                 include('event_new.html');
@@ -436,36 +522,46 @@
                     use_html_editor("description");
                 }
             }
-            print_side_block_end();
+
         break;
     }
     echo '</td>';
 
     // START: Last column (3-month display)
-    echo '<td style="vertical-align: top; width: 180px;">';
 
     $defaultcourses = calendar_get_default_courses();
-    calendar_set_filters($courses, $groups, $users, $defaultcourses, $defaultcourses);
-
-    print_side_block_start(get_string('monthlyview', 'calendar'));
+    //calendar_set_filters($courses, $groups, $users, $defaultcourses, $defaultcourses);
+    
+    // when adding an event you can not be a guest, so I think it's reasonalbe to ignore defaultcourses
+    // MDL-10353
+    calendar_set_filters($courses, $groups, $users);
     list($prevmon, $prevyr) = calendar_sub_month($mon, $yr);
     list($nextmon, $nextyr) = calendar_add_month($mon, $yr);
 
-    echo calendar_filter_controls('event', 'action='.$_REQUEST['action'].'&amp;type='.$_REQUEST['type'].'&amp;id='.$_REQUEST['id']);
-    echo '<p>';
-    echo calendar_top_controls('display', array('m' => $prevmon, 'y' => $prevyr));
-    echo calendar_get_mini($courses, $groups, $users, $prevmon, $prevyr);
-    echo '</p><p>';
-    echo calendar_top_controls('display', array('m' => $mon, 'y' => $yr));
-    echo calendar_get_mini($courses, $groups, $users, $mon, $yr);
-    echo '</p><p>';
-    echo calendar_top_controls('display', array('m' => $nextmon, 'y' => $nextyr));
-    echo calendar_get_mini($courses, $groups, $users, $nextmon, $nextyr);
-    echo '</p>';
-    print_side_block_end();
-    print_spacer(1, 180);
-    echo '</td>';
+    echo '<td class="sidecalendar">';
+    echo '<div class="sideblock">';
+    echo '<div class="header"><h2>'.get_string('eventskey', 'calendar').'</h2></div>';
+    echo '<div class="filters">';
+    echo calendar_filter_controls('event', 'action='.$action.'&amp;type='.$eventtype.'&amp;id='.$eventid);
+    echo '</div>';
+    echo '</div>';
 
+    echo '<div class="sideblock">';
+    echo '<div class="header"><h2>'.get_string('monthlyview', 'calendar').'</h2></div>';
+
+    echo '<div class="minicalendarblock minicalendartop">';
+    echo calendar_top_controls('display', array('id' => $urlcourse, 'm' => $prevmon, 'y' => $prevyr));
+    echo calendar_get_mini($courses, $groups, $users, $prevmon, $prevyr);
+    echo '</div><div class="minicalendarblock">';
+    echo calendar_top_controls('display', array('id' => $urlcourse, 'm' => $mon, 'y' => $yr));
+    echo calendar_get_mini($courses, $groups, $users, $mon, $yr);
+    echo '</div><div class="minicalendarblock">';
+    echo calendar_top_controls('display', array('id' => $urlcourse, 'm' => $nextmon, 'y' => $nextyr));
+    echo calendar_get_mini($courses, $groups, $users, $nextmon, $nextyr);
+    echo '</div>';
+    echo '</div>';
+
+    echo '</td>';
     echo '</tr></table>';
 
     print_footer();
@@ -479,9 +575,11 @@ function validate_form(&$form, &$err) {
     if(empty($form->name)) {
         $err['name'] = get_string('errornoeventname', 'calendar');
     }
+/* Allow events without a description
     if(empty($form->description)) {
         $err['description'] = get_string('errornodescription', 'calendar');
     }
+*/
     if(!checkdate($form->startmon, $form->startday, $form->startyr)) {
         $err['timestart'] = get_string('errorinvaliddate', 'calendar');
     }
@@ -506,46 +604,45 @@ function validate_form(&$form, &$err) {
     }
 }
 
-function calendar_add_event_allowed($courseid, $groupid, $userid) {
+function calendar_add_event_allowed($event) {
     global $USER;
 
-    if(isadmin()) {
-        return true;
+    // can not be using guest account
+    if (empty($USER->id) or $USER->username == 'guest') {
+        return false;
     }
-    else if($courseid == 0 && $groupid == 0 && $userid == $USER->id) {
-        return true;
-    }
-    else if($courseid != 0 && isteacheredit($courseid)) {
+
+    $sitecontext = get_context_instance(CONTEXT_SYSTEM);
+    // if user has manageentries at site level, always return true
+    if (has_capability('moodle/calendar:manageentries', $sitecontext)) {
         return true;
     }
 
-    return false;
+    switch ($event->type) {
+        case 'course':
+            return has_capability('moodle/calendar:manageentries', get_context_instance(CONTEXT_COURSE, $event->courseid));
+
+        case 'group':
+            // Allow users to add/edit group events if:
+            // 1) They have manageentries (= entries for whole course)
+            // 2) They have managegroupentries AND are in the group
+            $group = get_record('groups', 'id', $event->groupid);         
+            return $group && (
+                has_capability('moodle/calendar:manageentries', get_context_instance(CONTEXT_COURSE, $group->courseid)) ||
+                (has_capability('moodle/calendar:managegroupentries', get_context_instance(CONTEXT_COURSE, $group->courseid))
+                    && groups_is_member($event->groupid)));
+
+        case 'user':
+            if ($event->userid == $USER->id) {
+                return (has_capability('moodle/calendar:manageownentries', $sitecontext));
+            }
+            //there is no 'break;' intentionally
+
+        case 'site':
+            return has_capability('moodle/calendar:manageentries', get_context_instance(CONTEXT_COURSE, SITEID));
+
+        default:
+            return false;
+    }
 }
-
-function calendar_get_allowed_types(&$allowed) {
-    global $USER, $CFG, $SESSION;
-
-    $allowed->user = true; // User events always allowed
-    $allowed->groups = false; // This may change just below
-    $allowed->courses = false; // This may change just below
-    $allowed->site = isadmin($USER->id);
-
-    if(!empty($SESSION->cal_course_referer) && $SESSION->cal_course_referer > 1 && isteacheredit($SESSION->cal_course_referer, $USER->id)) {
-        $allowed->courses = array($SESSION->cal_course_referer => 1);
-        $allowed->groups = get_groups($SESSION->cal_course_referer);
-    }
-
-    //[pj]: This was used when we wanted to display all legal choices
-    /*
-    if($allowed->site) {
-        $allowed->courses = get_courses('all', 'c.shortname');
-        $allowed->groups = get_records_sql('SELECT g.*, c.fullname FROM '.$CFG->prefix.'groups g LEFT JOIN '.$CFG->prefix.'course c ON g.courseid = c.id ORDER BY c.shortname');
-    }
-    else if(!empty($USER->teacheredit)) {
-        $allowed->courses = get_records_select('course', 'id != 1 AND id IN ('.implode(',', array_keys($USER->teacheredit)).')');
-        $allowed->groups = get_records_sql('SELECT g.*, c.fullname FROM '.$CFG->prefix.'groups g LEFT JOIN '.$CFG->prefix.'course c ON g.courseid = c.id WHERE g.courseid IN ('.implode(',', array_keys($USER->teacheredit)).')');
-    }
-    */
-}
-
 ?>
