@@ -1,4 +1,4 @@
-<?php // $Id: view.php,v 1.69 2005/08/23 20:43:32 defacer Exp $
+<?php // $Id: view.php,v 1.95.2.10 2009/10/01 07:02:15 moodler Exp $
 
 /////////////////////////////////////////////////////////////////////////////
 //                                                                         //
@@ -9,14 +9,14 @@
 // Copyright (C) 2003-2004  Greek School Network            www.sch.gr     //
 //                                                                         //
 // Designed by:                                                            //
-//     Avgoustos Tsinakos (tsinakos@uom.gr)                                //
-//     Jon Papaioannou (pj@uom.gr)                                         //
+//     Avgoustos Tsinakos (tsinakos@teikav.edu.gr)                         //
+//     Jon Papaioannou (pj@moodle.org)                                     //
 //                                                                         //
 // Programming and development:                                            //
-//     Jon Papaioannou (pj@uom.gr)                                         //
+//     Jon Papaioannou (pj@moodle.org)                                     //
 //                                                                         //
 // For bugs, suggestions, etc contact:                                     //
-//     Jon Papaioannou (pj@uom.gr)                                         //
+//     Jon Papaioannou (pj@moodle.org)                                     //
 //                                                                         //
 // The current module was developed at the University of Macedonia         //
 // (www.uom.gr) under the funding of the Greek School Network (www.sch.gr) //
@@ -44,8 +44,8 @@
     require_once($CFG->dirroot.'/course/lib.php');
     require_once($CFG->dirroot.'/calendar/lib.php');
 
-    $course = optional_param('course', 0);
-    $view = optional_param('view', 'upcoming');
+    $courseid = optional_param('course', 0, PARAM_INT);
+    $view = optional_param('view', 'upcoming', PARAM_ALPHA);
     $day  = optional_param('cal_d', 0, PARAM_INT);
     $mon  = optional_param('cal_m', 0, PARAM_INT);
     $yr   = optional_param('cal_y', 0, PARAM_INT);
@@ -54,13 +54,27 @@
         redirect($CFG->wwwroot.'/'.$CFG->admin.'/index.php');
     }
 
+    if ($courseid && $courseid != SITEID) {
+        require_login($courseid);
+    } else if ($CFG->forcelogin) {
+        require_login();
+    }
+
+    // Initialize the session variables
+    calendar_session_vars();
+
     //add_to_log($course->id, "course", "view", "view.php?id=$course->id", "$course->id");
     $now = usergetdate(time());
     $pagetitle = '';
 
-    $nav = calendar_get_link_tag(get_string('calendar', 'calendar'), CALENDAR_URL.'view.php?view=upcoming&amp;', $now['mday'], $now['mon'], $now['year']);
+    $strcalendar = get_string('calendar', 'calendar');
+    $navlinks = array();
+    $navlinks[] = array('name' => $strcalendar,
+                        'link' =>calendar_get_link_href(CALENDAR_URL.'view.php?view=upcoming&amp;course='.$courseid.'&amp;',
+                                                        $now['mday'], $now['mon'], $now['year']),
+                        'type' => 'misc');
+
 
-    
     if(!checkdate($mon, $day, $yr)) {
         $day = intval($now['mday']);
         $mon = intval($now['mon']);
@@ -70,11 +84,11 @@
 
     switch($view) {
         case 'day':
-            $nav .= ' -> '.userdate($time, get_string('strftimedate'));
+            $navlinks[] = array('name' => userdate($time, get_string('strftimedate')), 'link' => null, 'type' => 'misc');
             $pagetitle = get_string('dayview', 'calendar');
         break;
         case 'month':
-            $nav .= ' -> '.userdate($time, get_string('strftimemonthyear'));
+            $navlinks[] = array('name' => userdate($time, get_string('strftimemonthyear')), 'link' => null, 'type' => 'misc');
             $pagetitle = get_string('detailedmonthview', 'calendar');
         break;
         case 'upcoming':
@@ -83,8 +97,8 @@
     }
 
     // If a course has been supplied in the URL, change the filters to show that one
-    if (!empty($course)) {
-        if ($course = get_record('course', 'id', $course)) {
+    if (!empty($courseid)) {
+        if ($course = get_record('course', 'id', $courseid)) {
             if ($course->id == SITEID) {
                 // If coming from the home page, show all courses
                 $SESSION->cal_courses_shown = calendar_get_default_courses(true);
@@ -96,6 +110,8 @@
                 calendar_set_referring_course($SESSION->cal_courses_shown);
             }
         }
+    } else {
+        $course = null;
     }
 
     if (empty($USER->id) or isguest()) {
@@ -110,9 +126,7 @@
     // but NOT for the "main page" course
     if ($SESSION->cal_course_referer != SITEID &&
        ($shortname = get_field('course', 'shortname', 'id', $SESSION->cal_course_referer)) !== false) {
-        // If we know about the referring course, show a return link and ALSO require login!
         require_login();
-        $nav = '<a href="'.$CFG->wwwroot.'/course/view.php?id='.$SESSION->cal_course_referer.'">'.$shortname.'</a> -> '.$nav;
         if (empty($course)) {
             $course = get_record('course', 'id', $SESSION->cal_course_referer); // Useful to have around
         }
@@ -122,57 +136,83 @@
     $prefsbutton = calendar_preferences_button();
 
     // Print title and header
-    print_header("$site->shortname: $strcalendar: $pagetitle", $strcalendar, $nav,
+    $navigation = build_navigation($navlinks);
+    print_header("$site->shortname: $strcalendar: $pagetitle", $strcalendar, $navigation,
                  '', '', true, $prefsbutton, user_login_string($site));
 
     echo calendar_overlib_html();
 
     // Layout the whole page as three big columns.
-    echo '<table id="calendar">';
+    echo '<table id="calendar" style="height:100%;">';
     echo '<tr>';
 
     // START: Main column
 
     echo '<td class="maincalendar">';
+    echo '<div class="heightcontainer">';
 
     switch($view) {
         case 'day':
-            calendar_show_day($day, $mon, $yr, $courses, $groups, $users);
+            calendar_show_day($day, $mon, $yr, $courses, $groups, $users, $courseid);
         break;
         case 'month':
-            calendar_show_month_detailed($mon, $yr, $courses, $groups, $users);
+            calendar_show_month_detailed($mon, $yr, $courses, $groups, $users, $courseid);
         break;
         case 'upcoming':
-            calendar_show_upcoming_events($courses, $groups, $users, get_user_preferences('calendar_lookahead', CALENDAR_UPCOMING_DAYS), get_user_preferences('calendar_maxevents', CALENDAR_UPCOMING_MAXEVENTS));
+            calendar_show_upcoming_events($courses, $groups, $users, get_user_preferences('calendar_lookahead', CALENDAR_UPCOMING_DAYS), get_user_preferences('calendar_maxevents', CALENDAR_UPCOMING_MAXEVENTS), $courseid);
         break;
     }
 
+    //Link to calendar export page
+    echo '<div class="bottom">';
+    if (!empty($CFG->enablecalendarexport)) {
+        print_single_button('export.php', array('course'=>$courseid), get_string('exportcalendar', 'calendar'));
+
+        if (!empty($USER->id)) {
+            $authtoken = sha1($USER->username . $USER->password . $CFG->calendar_exportsalt);
+            $usernameencoded = urlencode($USER->username);
+
+            echo "<a href=\"export_execute.php?preset_what=all&amp;preset_time=recentupcoming&amp;username=$usernameencoded&amp;authtoken=$authtoken\">"
+                 .'<img src="'.$CFG->pixpath.'/i/ical.gif" height="14" width="36" '
+                 .'alt="'.get_string('ical', 'calendar').'" '
+                 .'title="'.get_string('quickdownloadcalendar', 'calendar').'" />'
+                 .'</a>';
+        }
+    }
+
+    echo '</div>';
+    echo '</div>';
     echo '</td>';
 
     // END: Main column
 
     // START: Last column (3-month display)
     echo '<td class="sidecalendar">';
-    echo '<div class="header">'.get_string('monthlyview', 'calendar').'</div>';
-
     list($prevmon, $prevyr) = calendar_sub_month($mon, $yr);
     list($nextmon, $nextyr) = calendar_add_month($mon, $yr);
-    $getvars = 'cal_d='.$day.'&amp;cal_m='.$mon.'&amp;cal_y='.$yr; // For filtering
+    $getvars = 'id='.$courseid.'&amp;cal_d='.$day.'&amp;cal_m='.$mon.'&amp;cal_y='.$yr; // For filtering
 
+    echo '<div class="sideblock">';
+    echo '<div class="header"><h2>'.get_string('eventskey', 'calendar').'</h2></div>';
     echo '<div class="filters">';
-    echo calendar_filter_controls($view, $getvars);
+    echo calendar_filter_controls($view, $getvars, NULL, $courses);
+    echo '</div>';
     echo '</div>';
 
-    echo '<div class="minicalendarblock">';
-    echo calendar_top_controls('display', array('m' => $prevmon, 'y' => $prevyr));
+    echo '<div class="sideblock">';
+    echo '<div class="header"><h2>'.get_string('monthlyview', 'calendar').'</h2></div>';
+
+    echo '<div class="minicalendarblock minicalendartop">';
+    echo calendar_top_controls('display', array('id' => $courseid, 'm' => $prevmon, 'y' => $prevyr));
     echo calendar_get_mini($courses, $groups, $users, $prevmon, $prevyr);
     echo '</div><div class="minicalendarblock">';
-    echo calendar_top_controls('display', array('m' => $mon, 'y' => $yr));
+    echo calendar_top_controls('display', array('id' => $courseid, 'm' => $mon, 'y' => $yr));
     echo calendar_get_mini($courses, $groups, $users, $mon, $yr);
     echo '</div><div class="minicalendarblock">';
-    echo calendar_top_controls('display', array('m' => $nextmon, 'y' => $nextyr));
+    echo calendar_top_controls('display', array('id' => $courseid, 'm' => $nextmon, 'y' => $nextyr));
     echo calendar_get_mini($courses, $groups, $users, $nextmon, $nextyr);
     echo '</div>';
+    echo '</div>';
 
     echo '</td>';
 
@@ -182,7 +222,7 @@
 
 
 
-function calendar_show_day($d, $m, $y, $courses, $groups, $users) {
+function calendar_show_day($d, $m, $y, $courses, $groups, $users, $courseid) {
     global $CFG, $USER;
 
     if (!checkdate($m, $d, $y)) {
@@ -198,22 +238,24 @@ function calendar_show_day($d, $m, $y, $courses, $groups, $users) {
     $events = calendar_get_upcoming($courses, $groups, $users, 1, 100, $starttime);
 
     $text = '';
-    if (!isguest() && !empty($USER->id)) {
+    if (!isguest() && !empty($USER->id) && calendar_user_can_add_event()) {
         $text.= '<div class="buttons">';
         $text.= '<form action="'.CALENDAR_URL.'event.php" method="get">';
+        $text.= '<div>';
         $text.= '<input type="hidden" name="action" value="new" />';
+        $text.= '<input type="hidden" name="course" value="'.$courseid.'" />';
         $text.= '<input type="hidden" name="cal_d" value="'.$d.'" />';
         $text.= '<input type="hidden" name="cal_m" value="'.$m.'" />';
         $text.= '<input type="hidden" name="cal_y" value="'.$y.'" />';
         $text.= '<input type="submit" value="'.get_string('newevent', 'calendar').'" />';
-        $text.= '</form></div>';
+        $text.= '</div></form></div>';
     }
 
     $text .= get_string('dayview', 'calendar').': '.calendar_course_filter_selector($getvars);
 
     echo '<div class="header">'.$text.'</div>';
 
-    echo '<div class="controls">'.calendar_top_controls('day', array('d' => $d, 'm' => $m, 'y' => $y)).'</div>';
+    echo '<div class="controls">'.calendar_top_controls('day', array('id' => $courseid, 'd' => $d, 'm' => $m, 'y' => $y)).'</div>';
 
     if (empty($events)) {
         // There is nothing to display today.
@@ -227,6 +269,9 @@ function calendar_show_day($d, $m, $y, $courses, $groups, $users) {
 
         // First, print details about events that start today
         foreach ($events as $event) {
+
+            $event->calendarcourseid = $courseid;
+
             if ($event->timestart >= $starttime && $event->timestart <= $endtime) {  // Print it now
 
 
@@ -238,7 +283,8 @@ function calendar_show_day($d, $m, $y, $courses, $groups, $users) {
                 echo calendar_get_link_tag($dayend, CALENDAR_URL.'view.php?view=day'.$morehref.'&amp;', $enddate['mday'], $enddate['mon'], $enddate['year']).' ('.$timeend.')';
 */
                 //unset($event->time);
-                $event->time = calendar_format_event_time($event, time(), '', false);
+
+                $event->time = calendar_format_event_time($event, time(), '', false, $starttime);
                 calendar_print_event($event);
 
             } else {                                                                 // Save this for later
@@ -250,7 +296,7 @@ function calendar_show_day($d, $m, $y, $courses, $groups, $users) {
         if (!empty($underway)) {
             echo '<h3>'.get_string('spanningevents', 'calendar').':</h3>';
             foreach ($underway as $event) {
-                $event->time = calendar_format_event_time($event, time(), '', false);
+                $event->time = calendar_format_event_time($event, time(), '', false, $starttime);
                 calendar_print_event($event);
             }
         }
@@ -260,7 +306,7 @@ function calendar_show_day($d, $m, $y, $courses, $groups, $users) {
     }
 }
 
-function calendar_show_month_detailed($m, $y, $courses, $groups, $users) {
+function calendar_show_month_detailed($m, $y, $courses, $groups, $users, $courseid) {
     global $CFG, $SESSION, $USER, $CALENDARDAYS;
     global $day, $mon, $yr;
 
@@ -298,11 +344,18 @@ function calendar_show_month_detailed($m, $y, $courses, $groups, $users) {
     list($d, $m, $y) = array($date['mday'], $date['mon'], $date['year']); // This is what we want to display
     $display->maxdays = calendar_days_in_month($m, $y);
 
-    // We 'll keep these values as GMT here, and offset them when the time comes to query the db
-    $display->tstart = gmmktime(0, 0, 0, $m, 1, $y); // This is GMT
-    $display->tend = gmmktime(23, 59, 59, $m, $display->maxdays, $y); // GMT
-
-    $startwday = gmdate('w', $display->tstart); // $display->tstart is already GMT, so don't use date(): messes with server's TZ
+    $startwday = 0;
+    if (get_user_timezone_offset() < 99) {
+        // We 'll keep these values as GMT here, and offset them when the time comes to query the db
+        $display->tstart = gmmktime(0, 0, 0, $m, 1, $y); // This is GMT
+        $display->tend = gmmktime(23, 59, 59, $m, $display->maxdays, $y); // GMT
+        $startwday = gmdate('w', $display->tstart); // $display->tstart is already GMT, so don't use date(): messes with server's TZ
+    } else {
+        // no timezone info specified
+        $display->tstart = mktime(0, 0, 0, $m, 1, $y);
+        $display->tend = mktime(23, 59, 59, $m, $display->maxdays, $y);
+        $startwday = date('w', $display->tstart); // $display->tstart not necessarily GMT, so use date()
+    }
 
     // Align the starting weekday to fall in our display range
     if($startwday < $display->minwday) {
@@ -310,33 +363,42 @@ function calendar_show_month_detailed($m, $y, $courses, $groups, $users) {
     }
 
     // Get events from database
-    $whereclause = calendar_sql_where(usertime($display->tstart), usertime($display->tend), $users, $groups, $courses);
-    if($whereclause === false) {
-        $events = array();
+    $events = calendar_get_events(usertime($display->tstart), usertime($display->tend), $users, $groups, $courses);
+    if (!empty($events)) {
+        foreach($events as $eventid => $event) {
+            if (!empty($event->modulename)) {
+                $cm = get_coursemodule_from_instance($event->modulename, $event->instance);
+                if (!groups_course_module_visible($cm)) {
+                    unset($events[$eventid]);
+                }
+            }
+        }
     }
-    else {
-        $events = get_records_select('event', $whereclause, 'timestart');
-    }
-
+    
     // Extract information: events vs. time
-    calendar_events_by_day($events, $m, $y, $eventsbyday, $durationbyday, $typesbyday);
+    calendar_events_by_day($events, $m, $y, $eventsbyday, $durationbyday, $typesbyday, $courses);
 
     $text = '';
-    if(!isguest() && !empty($USER->id)) {
+    if(!isguest() && !empty($USER->id) && calendar_user_can_add_event()) {
         $text.= '<div class="buttons"><form action="'.CALENDAR_URL.'event.php" method="get">';
+        $text.= '<div>';
         $text.= '<input type="hidden" name="action" value="new" />';
+        $text.= '<input type="hidden" name="course" value="'.$courseid.'" />';
         $text.= '<input type="hidden" name="cal_m" value="'.$m.'" />';
         $text.= '<input type="hidden" name="cal_y" value="'.$y.'" />';
         $text.= '<input type="submit" value="'.get_string('newevent', 'calendar').'" />';
-        $text.= '</form></div>';
+        $text.= '</div></form></div>';
     }
 
-    $text .= get_string('detailedmonthview', 'calendar').': '.calendar_course_filter_selector($getvars);
+    $text .= '<label for="cal_course_flt_jump">'.
+               get_string('detailedmonthview', 'calendar').
+             ':</label>'.
+             calendar_course_filter_selector($getvars);
 
     echo '<div class="header">'.$text.'</div>';
 
     echo '<div class="controls">';
-    echo calendar_top_controls('month', array('m' => $m, 'y' => $y));
+    echo calendar_top_controls('month', array('id' => $courseid, 'm' => $m, 'y' => $y));
     echo '</div>';
 
     // Start calendar display
@@ -346,7 +408,7 @@ function calendar_show_month_detailed($m, $y, $courses, $groups, $users) {
     for($i = $display->minwday; $i <= $display->maxwday; ++$i) {
         // This uses the % operator to get the correct weekday no matter what shift we have
         // applied to the $display->minwday : $display->maxwday range from the default 0 : 6
-        echo '<th>'.get_string($CALENDARDAYS[$i % 7], 'calendar').'</th>';
+        echo '<th scope="col">'.get_string($CALENDARDAYS[$i % 7], 'calendar').'</th>';
     }
 
     echo '</tr><tr>'; // End of day names; prepare for day numbers
@@ -357,7 +419,7 @@ function calendar_show_month_detailed($m, $y, $courses, $groups, $users) {
 
     // Paddding (the first week may have blank days in the beginning)
     for($i = $display->minwday; $i < $startwday; ++$i) {
-        echo '<td>&nbsp;</td>'."\n";
+        echo '<td class="nottoday">&nbsp;</td>'."\n";
     }
 
     // Now display all the calendar
@@ -371,7 +433,7 @@ function calendar_show_month_detailed($m, $y, $courses, $groups, $users) {
 
         // Reset vars
         $cell = '';
-        $dayhref = calendar_get_link_href(CALENDAR_URL.'view.php?view=day&amp;', $day, $m, $y);
+        $dayhref = calendar_get_link_href(CALENDAR_URL.'view.php?view=day&amp;course='.$courseid.'&amp;', $day, $m, $y);
 
         if(CALENDAR_WEEKEND & (1 << ($dayweek % 7))) {
             // Weekend. This is true no matter what the exact range is.
@@ -384,18 +446,6 @@ function calendar_show_month_detailed($m, $y, $courses, $groups, $users) {
 
         // Special visual fx if an event is defined
         if(isset($eventsbyday[$day])) {
-            if(isset($typesbyday[$day]['startglobal'])) {
-                $class .= ' event_global';
-            }
-            else if(isset($typesbyday[$day]['startcourse'])) {
-                $class .= ' event_course';
-            }
-            else if(isset($typesbyday[$day]['startgroup'])) {
-                $class .= ' event_group';
-            }
-            else if(isset($typesbyday[$day]['startuser'])) {
-                $class .= ' event_user';
-            }
             if(count($eventsbyday[$day]) == 1) {
                 $title = get_string('oneevent', 'calendar');
             }
@@ -425,6 +475,8 @@ function calendar_show_month_detailed($m, $y, $courses, $groups, $users) {
         // Special visual fx for today
         if($display->thismonth && $day == $d) {
             $class .= ' today';
+        } else {
+            $class .= ' nottoday';
         }
 
         // Just display it
@@ -436,7 +488,14 @@ function calendar_show_month_detailed($m, $y, $courses, $groups, $users) {
         if(isset($eventsbyday[$day])) {
             echo '<ul class="events-new">';
             foreach($eventsbyday[$day] as $eventindex) {
-                echo '<li><a href="'.$dayhref.'">'.format_string($events[$eventindex]->name,true).'</a></li>';
+
+                // If event has a class set then add it to the event <li> tag
+                $eventclass = '';
+                if (!empty($events[$eventindex]->class)) {
+                    $eventclass = ' class="'.$events[$eventindex]->class.'"';
+                }
+
+                echo '<li'.$eventclass.'><a href="'.$dayhref.'#event_'.$events[$eventindex]->id.'">'.format_string($events[$eventindex]->name, true).'</a></li>';
             }
             echo '</ul>';
         }
@@ -452,80 +511,82 @@ function calendar_show_month_detailed($m, $y, $courses, $groups, $users) {
 
     // Paddding (the last week may have blank days at the end)
     for($i = $dayweek; $i <= $display->maxwday; ++$i) {
-        echo '<td>&nbsp;</td>';
+        echo '<td class="nottoday">&nbsp;</td>';
     }
     echo "</tr>\n"; // Last row ends
 
     echo "</table>\n"; // Tabular display of days ends
 
-    // OK, now for the filtering display
-    echo '<div class="filters"><table><tr>';
-
-    // Global events
-    if($SESSION->cal_show_global) {
-        echo '<td class="event_global" style="width: 8px;"></td><td><strong>'.get_string('globalevents', 'calendar').':</strong> ';
-        echo get_string('shown', 'calendar').' (<a href="'.CALENDAR_URL.'set.php?var=showglobal&amp;'.$getvars.'">'.get_string('clickhide', 'calendar').'</a>)</td>'."\n";
-    }
-    else {
-        echo '<td style="width: 8px;"></td><td><strong>'.get_string('globalevents', 'calendar').':</strong> ';
-        echo get_string('hidden', 'calendar').' (<a href="'.CALENDAR_URL.'set.php?var=showglobal&amp;'.$getvars.'">'.get_string('clickshow', 'calendar').'</a>)</td>'."\n";
-    }
-
-    // Course events
-    if(!empty($SESSION->cal_show_course)) {
-        echo '<td class="event_course" style="width: 8px;"></td><td><strong>'.get_string('courseevents', 'calendar').':</strong> ';
-        echo get_string('shown', 'calendar').' (<a href="'.CALENDAR_URL.'set.php?var=showcourses&amp;'.$getvars.'">'.get_string('clickhide', 'calendar').'</a>)</td>'."\n";
-    }
-    else {
-        echo '<td style="width: 8px;"></td><td><strong>'.get_string('courseevents', 'calendar').':</strong> ';
-        echo get_string('hidden', 'calendar').' (<a href="'.CALENDAR_URL.'set.php?var=showcourses&amp;'.$getvars.'">'.get_string('clickshow', 'calendar').'</a>)</td>'."\n";
-    }
-
-    echo "</tr>\n";
-
-    if(!empty($USER->id) && !isguest()) {
-        echo '<tr>';
-        // Group events
-        if($SESSION->cal_show_groups) {
-            echo '<td class="event_group" style="width: 8px;"></td><td><strong>'.get_string('groupevents', 'calendar').':</strong> ';
-            echo get_string('shown', 'calendar').' (<a href="'.CALENDAR_URL.'set.php?var=showgroups&amp;'.$getvars.'">'.get_string('clickhide', 'calendar').'</a>)</td>'."\n";
-        }
-        else {
-            echo '<td style="width: 8px;"></td><td><strong>'.get_string('groupevents', 'calendar').':</strong> ';
-            echo get_string('hidden', 'calendar').' (<a href="'.CALENDAR_URL.'set.php?var=showgroups&amp;'.$getvars.'">'.get_string('clickshow', 'calendar').'</a>)</td>'."\n";
-        }
-        // User events
-        if($SESSION->cal_show_user) {
-            echo '<td class="event_user" style="width: 8px;"></td><td><strong>'.get_string('userevents', 'calendar').':</strong> ';
-            echo get_string('shown', 'calendar').' (<a href="'.CALENDAR_URL.'set.php?var=showuser&amp;'.$getvars.'">'.get_string('clickhide', 'calendar').'</a>)</td>'."\n";
-        }
-        else {
-            echo '<td style="width: 8px;"></td><td><strong>'.get_string('userevents', 'calendar').':</strong> ';
-            echo get_string('hidden', 'calendar').' (<a href="'.CALENDAR_URL.'set.php?var=showuser&amp;'.$getvars.'">'.get_string('clickshow', 'calendar').'</a>)</td>'."\n";
-        }
-        echo "</tr>\n";
-    }
-
-    echo '</table></div>';
+	// OK, now for the filtering display 	 
+	     echo '<div class="filters"><table><tr>'; 	 
+	  	 
+	     // Global events 	 
+	     if($SESSION->cal_show_global) { 	 
+	         echo '<td class="event_global" style="width: 8px;"></td><td><strong>'.get_string('globalevents', 'calendar').':</strong> '; 	 
+	         echo get_string('shown', 'calendar').' (<a href="'.CALENDAR_URL.'set.php?var=showglobal&amp;'.$getvars.'">'.get_string('clickhide', 'calendar').'</a>)</td>'."\n"; 	 
+	     } 	 
+	     else { 	 
+	         echo '<td style="width: 8px;"></td><td><strong>'.get_string('globalevents', 'calendar').':</strong> '; 	 
+	         echo get_string('hidden', 'calendar').' (<a href="'.CALENDAR_URL.'set.php?var=showglobal&amp;'.$getvars.'">'.get_string('clickshow', 'calendar').'</a>)</td>'."\n"; 	 
+	     } 	 
+	  	 
+	     // Course events 	 
+	     if(!empty($SESSION->cal_show_course)) { 	 
+	         echo '<td class="event_course" style="width: 8px;"></td><td><strong>'.get_string('courseevents', 'calendar').':</strong> '; 	 
+	         echo get_string('shown', 'calendar').' (<a href="'.CALENDAR_URL.'set.php?var=showcourses&amp;'.$getvars.'">'.get_string('clickhide', 'calendar').'</a>)</td>'."\n"; 	 
+	     } 	 
+	     else { 	 
+	         echo '<td style="width: 8px;"></td><td><strong>'.get_string('courseevents', 'calendar').':</strong> '; 	 
+	         echo get_string('hidden', 'calendar').' (<a href="'.CALENDAR_URL.'set.php?var=showcourses&amp;'.$getvars.'">'.get_string('clickshow', 'calendar').'</a>)</td>'."\n"; 	 
+	     } 	 
+	  	 
+	     echo "</tr>\n"; 	 
+	  	 
+	     if(!empty($USER->id) && !isguest()) { 	 
+	         echo '<tr>'; 	 
+	         // Group events 	 
+	         if($SESSION->cal_show_groups) { 	 
+	             echo '<td class="event_group" style="width: 8px;"></td><td><strong>'.get_string('groupevents', 'calendar').':</strong> '; 	 
+	             echo get_string('shown', 'calendar').' (<a href="'.CALENDAR_URL.'set.php?var=showgroups&amp;'.$getvars.'">'.get_string('clickhide', 'calendar').'</a>)</td>'."\n"; 	 
+	         } 	 
+	         else { 	 
+	             echo '<td style="width: 8px;"></td><td><strong>'.get_string('groupevents', 'calendar').':</strong> '; 	 
+	             echo get_string('hidden', 'calendar').' (<a href="'.CALENDAR_URL.'set.php?var=showgroups&amp;'.$getvars.'">'.get_string('clickshow', 'calendar').'</a>)</td>'."\n"; 	 
+	         } 	 
+	         // User events 	 
+	         if($SESSION->cal_show_user) { 	 
+	             echo '<td class="event_user" style="width: 8px;"></td><td><strong>'.get_string('userevents', 'calendar').':</strong> '; 	 
+	             echo get_string('shown', 'calendar').' (<a href="'.CALENDAR_URL.'set.php?var=showuser&amp;'.$getvars.'">'.get_string('clickhide', 'calendar').'</a>)</td>'."\n"; 	 
+	         } 	 
+	         else { 	 
+	             echo '<td style="width: 8px;"></td><td><strong>'.get_string('userevents', 'calendar').':</strong> '; 	 
+	             echo get_string('hidden', 'calendar').' (<a href="'.CALENDAR_URL.'set.php?var=showuser&amp;'.$getvars.'">'.get_string('clickshow', 'calendar').'</a>)</td>'."\n"; 	 
+	         } 	 
+	         echo "</tr>\n"; 	 
+	     } 	 
+	  	 
+	     echo '</table></div>';
 }
 
-function calendar_show_upcoming_events($courses, $groups, $users, $futuredays, $maxevents) {
+function calendar_show_upcoming_events($courses, $groups, $users, $futuredays, $maxevents, $courseid) {
     global $USER;
 
     $events = calendar_get_upcoming($courses, $groups, $users, $futuredays, $maxevents);
 
     $text = '';
 
-    if(!isguest() && !empty($USER->id)) {
+    if(!isguest() && !empty($USER->id) && calendar_user_can_add_event()) {
         $text.= '<div class="buttons">';
         $text.= '<form action="'.CALENDAR_URL.'event.php" method="get">';
+        $text.= '<div>';
         $text.= '<input type="hidden" name="action" value="new" />';
+        $text.= '<input type="hidden" name="course" value="'.$courseid.'" />';
         /*
         $text.= '<input type="hidden" name="cal_m" value="'.$m.'" />';
         $text.= '<input type="hidden" name="cal_y" value="'.$y.'" />';
         */
         $text.= '<input type="submit" value="'.get_string('newevent', 'calendar').'" />';
-        $text.= '</form></div>';
+        $text.= '</div></form></div>';
     }
 
     $text .= get_string('upcomingevents', 'calendar').': '.calendar_course_filter_selector('from=upcoming');
@@ -533,8 +594,10 @@ function calendar_show_upcoming_events($courses, $groups, $users, $futuredays, $
     echo '<div class="header">'.$text.'</div>';
 
     if ($events) {
+
         echo '<div class="eventlist">';
         foreach ($events as $event) {
+            $event->calendarcourseid = $courseid;
             calendar_print_event($event);
         }
         echo '</div>';
@@ -550,7 +613,7 @@ function calendar_course_filter_selector($getvars = '') {
         return '';
     }
 
-    if (isadmin() && !empty($CFG->calendar_adminseesall)) {
+    if (has_capability('moodle/calendar:manageentries', get_context_instance(CONTEXT_SYSTEM)) && !empty($CFG->calendar_adminseesall)) {
         $courses = get_courses('all', 'c.shortname','c.id,c.shortname');
     } else {
         $courses = get_my_courses($USER->id, 'shortname');
@@ -560,7 +623,7 @@ function calendar_course_filter_selector($getvars = '') {
 
     $courseoptions[SITEID] = get_string('fulllistofcourses');
     foreach ($courses as $course) {
-        $courseoptions[$course->id] = $course->shortname;
+        $courseoptions[$course->id] = format_string($course->shortname);
     }
 
     if (is_numeric($SESSION->cal_courses_shown)) {
