@@ -11,15 +11,21 @@
     require_once($CFG->dirroot . '/course/lib.php');
     require_once($CFG->dirroot . '/lib/moodlelib.php');
     require_once($CFG->dirroot . '/lib/datalib.php');
-    require_once($CFG->dirroot . "/enrol/" . $CFG->enrol . "/enrol.php");
+    require_once($CFG->dirroot . "/enrol/ldap/enrol.php");
 
     // ensure errors are well explained
-    $CFG->debug=10;
+    $CFG->debug = DEBUG_NORMAL;
+
     // update enrolments -- these handlers should autocreate courses if required
-    $enrol = new enrolment_plugin();
-    $enrol->enrol_ldap_connect();    
-    $enrol->sync_enrolments('student', true);
-    $enrol->sync_enrolments('teacher', true);
+    $enrol = new enrolment_plugin_ldap();
+    $enrol->enrol_ldap_connect();
+
+    $enrol->check_legacy_config();
+
+    $roles = get_records('role');
+    foreach ($roles as $role) {
+        $enrol->sync_enrolments($role->shortname, true);
+    }
     
     // sync metacourses
     if (function_exists('sync_metacourses')) {
