@@ -15,12 +15,26 @@
 function censor_filter($courseid, $text) {
 
     static $words;
+    global $CFG;
+
+    if (!isset($CFG->filter_censor_badwords)) {
+        set_config( 'filter_censor_badwords','' );
+    }
 
     if (empty($words)) {
         $words = array();
-        $badwords = explode(',', get_string('badwords', 'censor'));
+        // if no user-specified words, use default list from language pack
+        if (empty($CFG->filter_censor_badwords)) {
+            $badwords = explode(',',get_string('badwords','censor') );
+        }
+        else {
+            $badwords = explode(',', $CFG->filter_censor_badwords );
+        }
         foreach ($badwords as $badword) {
-            $words[] = new filterobject(trim($badword), '<span class="censoredtext">', '</span>', false, false);
+            $badword = trim($badword);
+            // See MDL-3964 for explanation of leaving the badword in the span title 
+            $words[] = new filterobject($badword, '<span class="censoredtext" title="'.$badword.'">', '</span>', 
+                                        false, false, str_pad('',strlen($badword),'*'));
         }
     }
 
