@@ -1,4 +1,4 @@
-<?php  // $Id: excellib.class.php,v 1.4 2006/05/02 15:28:02 stronk7 Exp $
+<?php  // $Id: excellib.class.php,v 1.12.2.5 2009/11/12 01:58:05 fmarier Exp $
 
 ///////////////////////////////////////////////////////////////////////////
 //                                                                       //
@@ -7,7 +7,7 @@
 // Moodle - Modular Object-Oriented Dynamic Learning Environment         //
 //          http://moodle.com                                            //
 //                                                                       //
-// Copyright (C) 2001-3001 Martin Dougiamas        http://dougiamas.com  //
+// Copyright (C) 1999 onwards Martin Dougiamas     http://dougiamas.com  //
 //           (C) 2001-3001 Eloy Lafuente (stronk7) http://contiento.com  //
 //                                                                       //
 // This program is free software; you can redistribute it and/or modify  //
@@ -24,16 +24,12 @@
 //                                                                       //
 ///////////////////////////////////////////////////////////////////////////
 
-/// We need to add this to allow "our" PEAR package to work smoothly
-/// without modifying one bit, putting it in the 1st place of the
-/// include_path to be localised by Moodle without problems
-ini_set('include_path', $CFG->libdir.'/pear' . PATH_SEPARATOR . ini_get('include_path'));
-
+//setup.php icludes our hacked pear libs first
 require_once 'Spreadsheet/Excel/Writer.php';
 
 /**
 * Define and operate over one Moodle Workbook.
-* 
+*
 * A big part of this class acts as a wrapper over the PEAR
 * Spreadsheet_Excel_Writer_Workbook and OLE libraries
 * maintaining Moodle functions isolated from underlying code.
@@ -41,27 +37,43 @@ require_once 'Spreadsheet/Excel/Writer.php';
 class MoodleExcelWorkbook {
 
     var $pear_excel_workbook;
+    var $latin_output;
 
-    /* Constructs one Moodle Workbook.
+    /**
+     * Constructs one Moodle Workbook.
+     *
      * @param string $filename The name of the file
      */
     function MoodleExcelWorkbook($filename) {
+        global $CFG;
     /// Internally, create one PEAR Spreadsheet_Excel_Writer_Workbook class
         $this->pear_excel_workbook = new Spreadsheet_Excel_Writer($filename);
     /// Prepare it to accept UTF-16LE data and to encode it properly
-        $this->pear_excel_workbook->setVersion(8);
+        if (empty($CFG->latinexcelexport)) { /// Only if don't want to use latin (win1252) stronger output
+            $this->pear_excel_workbook->setVersion(8);
+            $this->latin_output = false;
+        } else { /// We want latin (win1252) output
+            $this->latin_output = true;
+        }
+    /// Choose our temporary directory - see MDL-7176, found by paulo.matos
+        make_upload_directory('temp/excel', false);
+        $this->pear_excel_workbook->setTempDir($CFG->dataroot.'/temp/excel');
     }
 
-    /* Create one Moodle Worksheet
+    /**
+     * Create one Moodle Worksheet
+     *
      * @param string $name Name of the sheet
      */
     function &add_worksheet($name = '') {
     /// Create the Moodle Worksheet. Returns one pointer to it
-        $ws =& new MoodleExcelWorksheet ($name, $this->pear_excel_workbook);
+        $ws =& new MoodleExcelWorksheet ($name, $this->pear_excel_workbook, $this->latin_output);
         return $ws;
     }
 
-    /* Create one Moodle Format
+    /**
+     * Create one Moodle Format
+     *
      * @param array $properties array of properties [name]=value;
      *                          valid names are set_XXXX existing
      *                          functions without the set_ part
@@ -73,13 +85,16 @@ class MoodleExcelWorkbook {
         return $ft;
     }
 
-    /* Close the Moodle Workbook
+    /**
+     * Close the Moodle Workbook
      */
     function close() {
         $this->pear_excel_workbook->close();
     }
 
-    /* Write the correct HTTP headers 
+    /**
+     * Write the correct HTTP headers
+     *
      * @param string $name Name of the downloaded file
      */
     function send($filename) {
@@ -89,7 +104,7 @@ class MoodleExcelWorkbook {
 
 /**
 * Define and operate over one Worksheet.
-* 
+*
 * A big part of this class acts as a wrapper over the PEAR
 * Spreadsheet_Excel_Writer_Workbook and OLE libraries
 * maintaining Moodle functions isolated from underlying code.
@@ -97,94 +112,134 @@ class MoodleExcelWorkbook {
 class MoodleExcelWorksheet {
 
     var $pear_excel_worksheet;
+    var $latin_output;
 
-    /* Constructs one Moodle Worksheet.
+    /**
+     * Constructs one Moodle Worksheet.
+     *
      * @param string $filename The name of the file
      * @param object $workbook The internal PEAR Workbook onject we are creating
      */
-    function MoodleExcelWorksheet($name, &$workbook) {
+    function MoodleExcelWorksheet($name, &$workbook, $latin_output=false) {
+
     /// Internally, add one sheet to the workbook    
         $this->pear_excel_worksheet =& $workbook->addWorksheet($name);
+        $this->latin_output = $latin_output;
     /// Set encoding to UTF-16LE 
-        $this->pear_excel_worksheet->setInputEncoding('UTF-16LE');
+        if (!$this->latin_output) { /// Only if don't want to use latin (win1252) stronger output
+            $this->pear_excel_worksheet->setInputEncoding('UTF-16LE');
+        }
     }
 
-    /* Write one string somewhere in the worksheet
+    /**
+     * Write one string somewhere in the worksheet
+     *
      * @param integer $row    Zero indexed row
      * @param integer $col    Zero indexed column
      * @param string  $str    The string to write
      * @param mixed   $format The XF format for the cell
      */
-    function write_string($row, $col, $str, $format=0) {
+    function write_string($row, $col, $str, $format=null) {
     /// Calculate the internal PEAR format
         $format = $this->MoodleExcelFormat2PearExcelFormat($format);
     /// Loading the textlib singleton instance. We are going to need it.
         $textlib = textlib_get_instance();
     /// Convert the text from its original encoding to UTF-16LE
-        $str = $textlib->convert($str, current_charset(), 'utf-16le');
+        if (!$this->latin_output) { /// Only if don't want to use latin (win1252) stronger output
+            $str = $textlib->convert($str, 'utf-8', 'utf-16le');
+        } else { /// else, convert to latin (win1252)
+            $str = $textlib->convert($str, 'utf-8', 'windows-1252');
+        }
     /// Add the string safely to the PEAR Worksheet
         $this->pear_excel_worksheet->writeString($row, $col, $str, $format);
     }
 
-    /* Write one number somewhere in the worksheet
+    /**
+     * Write one number somewhere in the worksheet
+     *
      * @param integer $row    Zero indexed row
      * @param integer $col    Zero indexed column
      * @param float   $num    The number to write
      * @param mixed   $format The XF format for the cell
      */
-    function write_number($row, $col, $num, $format=0) {
+    function write_number($row, $col, $num, $format=null) {
     /// Calculate the internal PEAR format
         $format = $this->MoodleExcelFormat2PearExcelFormat($format);
     /// Add  the number safely to the PEAR Worksheet
         $this->pear_excel_worksheet->writeNumber($row, $col, $num, $format);
     }
 
-    /* Write one url somewhere in the worksheet
+    /**
+     * Write one url somewhere in the worksheet
+     *
      * @param integer $row    Zero indexed row
      * @param integer $col    Zero indexed column
      * @param string  $url    The url to write
      * @param mixed   $format The XF format for the cell
      */
-    function write_url($row, $col, $url, $format=0) {
+    function write_url($row, $col, $url, $format=null) {
     /// Calculate the internal PEAR format
         $format = $this->MoodleExcelFormat2PearExcelFormat($format);
     /// Add  the url safely to the PEAR Worksheet
         $this->pear_excel_worksheet->writeUrl($row, $col, $url, $format);
     }
 
-    /* Write one formula somewhere in the worksheet
+    /**
+     * Write one date somewhere in the worksheet
+     * @param integer $row    Zero indexed row
+     * @param integer $col    Zero indexed column
+     * @param string  $date   The date to write in UNIX timestamp format
+     * @param mixed   $format The XF format for the cell
+     */
+    function write_date($row, $col, $date, $format=null) {
+    /// Calculate the internal PEAR format
+        $format = $this->MoodleExcelFormat2PearExcelFormat($format);
+    /// Convert the date to Excel format
+        $timezone = get_user_timezone_offset();
+        $value =  ((usertime($date) + (int)($timezone * HOURSECS * 2)) / 86400) + 25569;
+    /// Add  the date safely to the PEAR Worksheet
+        $this->pear_excel_worksheet->writeNumber($row, $col, $value, $format);
+    }
+
+    /**
+     * Write one formula somewhere in the worksheet
+     *
      * @param integer $row    Zero indexed row
      * @param integer $col    Zero indexed column
      * @param string  $formula The formula to write
      * @param mixed   $format The XF format for the cell
      */
-    function write_formula($row, $col, $formula, $format=0) {
+    function write_formula($row, $col, $formula, $format=null) {
     /// Calculate the internal PEAR format
         $format = $this->MoodleExcelFormat2PearExcelFormat($format);
     /// Add  the formula safely to the PEAR Worksheet
         $this->pear_excel_worksheet->writeFormula($row, $col, $formula, $format);
     }
 
-    /* Write one blanck somewhere in the worksheet
+    /**
+     * Write one blanck somewhere in the worksheet
+     *
      * @param integer $row    Zero indexed row
      * @param integer $col    Zero indexed column
      * @param mixed   $format The XF format for the cell
      */
-    function write_blank($row, $col, $format=0) {
+    function write_blank($row, $col, $format=null) {
     /// Calculate the internal PEAR format
         $format = $this->MoodleExcelFormat2PearExcelFormat($format);
     /// Add  the blank safely to the PEAR Worksheet
         $this->pear_excel_worksheet->writeBlank($row, $col, $format);
     }
 
-    /* Write anything somewhere in the worksheet
+    /**
+     * Write anything somewhere in the worksheet
      * Type will be automatically detected
+     *
      * @param integer $row    Zero indexed row
      * @param integer $col    Zero indexed column
      * @param mixed   $token  What we are writing
      * @param mixed   $format The XF format for the cell
      */
-    function write($row, $col, $token, $format=0) {
+    function write($row, $col, $token, $format=null) {
 
     /// Analyse what are we trying to send
         if (preg_match("/^([+-]?)(?=\d|\.\d)\d*(\.\d*)?([Ee]([+-]?\d+))?$/", $token)) {
@@ -214,21 +269,25 @@ class MoodleExcelWorksheet {
         }
     }
 
-    /* Sets the height (and other settings) of one row
+    /**
+     * Sets the height (and other settings) of one row
+     *
      * @param integer $row    The row to set
      * @param integer $height Height we are giving to the row (null to set just format withouth setting the height)
      * @param mixed   $format The optional XF format we are giving to the row
      * @param bool    $hidden The optional hidden attribute
      * @param integer $level  The optional outline level (0-7)
      */
-    function set_row ($row, $height, $format = 0, $hidden = false, $level = 0) {
+    function set_row ($row, $height, $format = null, $hidden = false, $level = 0) {
     /// Calculate the internal PEAR format
         $format = $this->MoodleExcelFormat2PearExcelFormat($format);
     /// Set the row safely to the PEAR Worksheet
         $this->pear_excel_worksheet->setRow($row, $height, $format, $hidden, $level);
     }
 
-    /* Sets the width (and other settings) of one column
+    /**
+     * Sets the width (and other settings) of one column
+     *
      * @param integer $firstcol first column on the range
      * @param integer $lastcol  last column on the range
      * @param integer $width    width to set
@@ -236,22 +295,75 @@ class MoodleExcelWorksheet {
      * @param integer $hidden   The optional hidden atribute
      * @param integer $level    The optional outline level (0-7)
      */
-    function set_column ($firstcol, $lastcol, $width, $format = 0, $hidden = false, $level = 0) {
+    function set_column ($firstcol, $lastcol, $width, $format = null, $hidden = false, $level = 0) {
     /// Calculate the internal PEAR format
         $format = $this->MoodleExcelFormat2PearExcelFormat($format);
     /// Set the column safely to the PEAR Worksheet
         $this->pear_excel_worksheet->setColumn($firstcol, $lastcol, $width, $format, $hidden, $level);
     }
 
-    /* Returns the PEAR Excel Format for one Moodle Excel Format
+    /**
+    * Set the option to hide gridlines on the printed page.
+    *
+    * @access public
+    */
+    function hide_gridlines() {
+        $this->pear_excel_worksheet->hideGridLines();
+    }
+
+    /**
+    * Set the option to hide gridlines on the worksheet (as seen on the screen).
+    *
+    * @access public
+    */
+    function hide_screen_gridlines() {
+        $this->pear_excel_worksheet->hideScreenGridlines();
+    }
+    
+    /**
+    * Insert a 24bit bitmap image in a worksheet.
+    *
+    * @access public
+    * @param integer $row     The row we are going to insert the bitmap into
+    * @param integer $col     The column we are going to insert the bitmap into
+    * @param string  $bitmap  The bitmap filename
+    * @param integer $x       The horizontal position (offset) of the image inside the cell.
+    * @param integer $y       The vertical position (offset) of the image inside the cell.
+    * @param integer $scale_x The horizontal scale
+    * @param integer $scale_y The vertical scale
+    */
+    function insert_bitmap($row, $col, $bitmap, $x = 0, $y = 0, $scale_x = 1, $scale_y = 1) {
+    /// Add the bitmap safely to the PEAR Worksheet
+        $this->pear_excel_worksheet->insertBitmap($row, $col, $bitmap, $x, $y, $scale_x, $scale_y);
+    }
+
+    /**
+    * Merges the area given by its arguments.
+    * This is an Excel97/2000 method. It is required to perform more complicated
+    * merging than the normal setAlign('merge').
+    *
+    * @access public
+    * @param integer $first_row First row of the area to merge
+    * @param integer $first_col First column of the area to merge
+    * @param integer $last_row  Last row of the area to merge
+    * @param integer $last_col  Last column of the area to merge
+    */
+    function merge_cells($first_row, $first_col, $last_row, $last_col) {
+        /// Merge cells safely to the PEAR Worksheet
+        $this->pear_excel_worksheet->mergeCells($first_row, $first_col, $last_row, $last_col);
+    }
+
+    /**
+     * Returns the PEAR Excel Format for one Moodle Excel Format
+     *
      * @param mixed MoodleExcelFormat object
      * @return mixed PEAR Excel Format object
      */
     function MoodleExcelFormat2PearExcelFormat($format) {
-        if (is_object($format)) {
+        if ($format) {
             return $format->pear_excel_format;
         } else {
-            return 0;
+            return null;
         }
     }
 }
@@ -259,7 +371,7 @@ class MoodleExcelWorksheet {
 
 /**
 * Define and operate over one Format.
-* 
+*
 * A big part of this class acts as a wrapper over the PEAR
 * Spreadsheet_Excel_Writer_Workbook and OLE libraries
 * maintaining Moodle functions isolated from underlying code.
@@ -268,7 +380,9 @@ class MoodleExcelFormat {
 
     var $pear_excel_format;
 
-    /* Constructs one Moodle Format.
+    /**
+     * Constructs one Moodle Format.
+     *
      * @param object $workbook The internal PEAR Workbook onject we are creating
      */
     function MoodleExcelFormat(&$workbook, $properties = array()) {
@@ -283,7 +397,20 @@ class MoodleExcelFormat {
         }
     }
 
-    /* Set weight of the format
+    /**
+     * Set the size of the text in the format (in pixels).
+     * By default all texts in generated sheets are 10px.
+     *
+     * @param integer $size Size of the text (in pixels)
+     */
+    function set_size($size) {
+    /// Set the size safely to the PEAR Format
+        $this->pear_excel_format->setSize($size);
+    }
+
+    /**
+     * Set weight of the format
+     *
      * @param integer $weight Weight for the text, 0 maps to 400 (normal text),
      *                        1 maps to 700 (bold text). Valid range is: 100-1000.
      *                        It's Optional, default is 1 (bold).
@@ -293,7 +420,9 @@ class MoodleExcelFormat {
         $this->pear_excel_format->setBold($weight);
     }
 
-    /* Set underline of the format
+    /**
+     * Set underline of the format
+     *
      * @param integer $underline The value for underline. Possible values are:
      *                           1 => underline, 2 => double underline
      */
@@ -302,35 +431,41 @@ class MoodleExcelFormat {
         $this->pear_excel_format->setUnderline($underline);
     }
 
-    /* Set italic of the format
+    /**
+     * Set italic of the format
      */
     function set_italic() {
     /// Set the italic safely to the PEAR Format
         $this->pear_excel_format->setItalic();
     }
 
-    /* Set strikeout of the format
+    /**
+     * Set strikeout of the format
      */
     function set_strikeout() {
     /// Set the strikeout safely to the PEAR Format
         $this->pear_excel_format->setStrikeOut();
     }
 
-    /* Set outlining of the format
+    /**
+     * Set outlining of the format
      */
     function set_outline() {
     /// Set the outlining safely to the PEAR Format
         $this->pear_excel_format->setOutLine();
     }
 
-    /* Set shadow of the format
+    /**
+     * Set shadow of the format
      */
     function set_shadow() {
     /// Set the shadow safely to the PEAR Format
         $this->pear_excel_format->setShadow();
     }
 
-    /* Set the script of the text
+    /**
+     * Set the script of the text
+     *
      * @param integer $script The value for script type. Possible values are:
      *                        1 => superscript, 2 => subscript
      */
@@ -339,7 +474,9 @@ class MoodleExcelFormat {
         $this->pear_excel_format->setScript($script);
     }
 
-    /* Set color of the format
+    /**
+     * Set color of the format. Used to specify the color of the text to be formatted.
+     *
      * @param mixed $color either a string (like 'blue'), or an integer (range is [8...63])
      */
     function set_color($color) {
@@ -347,7 +484,12 @@ class MoodleExcelFormat {
         $this->pear_excel_format->setColor($color);
     }
 
-    /* Set foreground color of the format
+    /**
+     * Set foreground color (top layer) of the format. About formatting colors note that cells backgrounds
+     * have TWO layers, in order to support patterns and paint them with two diferent colors.
+     * This method set the color of the TOP layer of the background format. So, when filling
+     * cells with plain colors (no patterns) this is the method to use.
+     *
      * @param mixed $color either a string (like 'blue'), or an integer (range is [8...63])
      */
     function set_fg_color($color) {
@@ -355,7 +497,13 @@ class MoodleExcelFormat {
         $this->pear_excel_format->setFgColor($color);
     }
 
-    /* Set background color of the format
+    /**
+     * Set background color (bottom layer) of the format. About formatting colors note that cells backgrounds
+     * have TWO layers, in order to support patterns and paint them with two diferent colors.
+     * This method set the color of the BOTTOM layer of the background format. So, the color
+     * specified here only will be visible if using patterns. Use set_fg_color() to fill
+     * cells with plain colors (no patterns).
+     *
      * @param mixed $color either a string (like 'blue'), or an integer (range is [8...63])
      */
     function set_bg_color($color) {
@@ -363,7 +511,8 @@ class MoodleExcelFormat {
         $this->pear_excel_format->setBgColor($color);
     }
 
-    /* Set the fill pattern of the format
+    /**
+     * Set the fill pattern of the format
      * @param integer Optional. Defaults to 1. Meaningful values are: 0-18
      *                0 meaning no background.
      */
@@ -372,14 +521,17 @@ class MoodleExcelFormat {
         $this->pear_excel_format->setPattern($pattern);
     }
 
-    /* Set text wrap of the format
+    /**
+     * Set text wrap of the format
      */
     function set_text_wrap() {
     /// Set the shadow safely to the PEAR Format
         $this->pear_excel_format->setTextWrap();
     }
 
-    /* Set the cell alignment of the format
+    /**
+     * Set the cell alignment of the format
+     *
      * @param string $location alignment for the cell ('left', 'right', etc...)
      */
     function set_align($location) {
@@ -387,7 +539,9 @@ class MoodleExcelFormat {
         $this->pear_excel_format->setAlign($location);
     }
 
-    /* Set the cell horizontal alignment of the format
+    /**
+     * Set the cell horizontal alignment of the format
+     *
      * @param string $location alignment for the cell ('left', 'right', etc...)
      */
     function set_h_align($location) {
@@ -395,7 +549,9 @@ class MoodleExcelFormat {
         $this->pear_excel_format->setHAlign($location);
     }
 
-    /* Set the cell vertical alignment of the format
+    /**
+     * Set the cell vertical alignment of the format
+     *
      * @param string $location alignment for the cell ('top', 'vleft', etc...)
      */
     function set_v_align($location) {
@@ -403,7 +559,9 @@ class MoodleExcelFormat {
         $this->pear_excel_format->setVAlign($location);
     }
 
-    /* Set the top border of the format
+    /**
+     * Set the top border of the format
+     *
      * @param integer $style style for the cell. 1 => thin, 2 => thick
      */
     function set_top($style) {
@@ -411,7 +569,9 @@ class MoodleExcelFormat {
         $this->pear_excel_format->setTop($style);
     }
 
-    /* Set the bottom border of the format
+    /**
+     * Set the bottom border of the format
+     *
      * @param integer $style style for the cell. 1 => thin, 2 => thick
      */
     function set_bottom($style) {
@@ -419,7 +579,9 @@ class MoodleExcelFormat {
         $this->pear_excel_format->setBottom($style);
     }
 
-    /* Set the left border of the format
+    /**
+     * Set the left border of the format
+     *
      * @param integer $style style for the cell. 1 => thin, 2 => thick
      */
     function set_left($style) {
@@ -427,7 +589,9 @@ class MoodleExcelFormat {
         $this->pear_excel_format->setLeft($style);
     }
 
-    /* Set the right border of the format
+    /**
+     * Set the right border of the format
+     *
      * @param integer $style style for the cell. 1 => thin, 2 => thick
      */
     function set_right($style) {
@@ -437,6 +601,7 @@ class MoodleExcelFormat {
 
     /**
      * Set cells borders to the same style
+     *
      * @param integer $style style to apply for all cell borders. 1 => thin, 2 => thick.
      */
     function set_border($style) {
@@ -444,10 +609,10 @@ class MoodleExcelFormat {
         $this->pear_excel_format->setBorder($style);
     }
 
-    /* Set the numerical format of the format
-     * It can be date, time, currency, etc...
-    /* Set the numerical format of the format
+    /**
+     * Set the numerical format of the format
      * It can be date, time, currency, etc...
+     *
      * @param integer $num_format The numeric format
      */
     function set_num_format($num_format) {
