@@ -1,14 +1,13 @@
-<?php  // $Id: view.php,v 1.39 2005/04/21 08:41:06 moodler Exp $
+<?php  // $Id: view.php,v 1.42 2007/08/17 12:15:33 skodak Exp $
 
     require_once("../../config.php");
     require_once("lib.php");
  
-    $id = optional_param('id');    // Course Module ID
-    $a  = optional_param('a');
-
+    $id = optional_param('id', 0, PARAM_INT);  // Course Module ID
+    $a  = optional_param('a', 0, PARAM_INT);   // Assignment ID
 
     if ($id) {
-        if (! $cm = get_record("course_modules", "id", $id)) {
+        if (! $cm = get_coursemodule_from_id('assignment', $id)) {
             error("Course Module ID was incorrect");
         }
 
@@ -31,7 +30,7 @@
         }
     }
 
-    require_login($course->id);
+    require_login($course, true, $cm);
 
     require ("$CFG->dirroot/mod/assignment/type/$assignment->assignmenttype/assignment.class.php");
     $assignmentclass = "assignment_$assignment->assignmenttype";
