@@ -17,7 +17,7 @@ define('CHAT_SIDEKICK_MESSAGE',     0x22);
 define('CHAT_SIDEKICK_BEEP',        0x23);
 
 $phpversion = phpversion();
-echo 'Moodle chat daemon v1.0 on PHP '.$phpversion." (\$Id: chatd.php,v 1.29 2005/04/01 05:55:10 martinlanghoff Exp $)\n\n";
+echo 'Moodle chat daemon v1.0 on PHP '.$phpversion." (\$Id: chatd.php,v 1.32.4.6 2009/11/19 10:35:17 skodak Exp $)\n\n";
 
 /// Set up all the variables we need   /////////////////////////////////////
 
@@ -27,6 +27,8 @@ $_SERVER['PHP_SELF']        = 'dummy';
 $_SERVER['SERVER_NAME']     = 'dummy';
 $_SERVER['HTTP_USER_AGENT'] = 'dummy';
 
+$nomoodlecookie = true;
+
 include('../../config.php');
 include('lib.php');
 
@@ -42,6 +44,11 @@ if(!empty($safemode)) {
     die("Error: Cannot run with PHP safe_mode = On. Turn off safe_mode in php.ini.\n");
 }
 
+$passref = ini_get('allow_call_time_pass_reference');
+if(empty($passref)) {
+    die("Error: Cannot run with PHP allow_call_time_pass_reference = Off. Turn on allow_call_time_pass_reference in php.ini.\n");
+}
+
 @set_time_limit (0);
 set_magic_quotes_runtime(0);
 error_reporting(E_ALL);
@@ -65,7 +72,7 @@ class ChatConnection {
 
     function ChatConnection($resource) {
         $this->handle = $resource;
-        socket_getpeername($this->handle, &$this->ip, &$this->port);
+        @socket_getpeername($this->handle, $this->ip, $this->port);
     }
 }
 
@@ -222,7 +229,7 @@ class ChatDaemon {
         static $str;
 
         $info = &$this->sets_info[$sessionid];
-        chat_force_language($info['lang']);
+        course_setup($info['course'], $info['user']);
 
         $timenow = time();
 
@@ -237,26 +244,30 @@ class ChatDaemon {
             $str->mins  = get_string("mins");
             $str->sec   = get_string("sec");
             $str->secs  = get_string("secs");
+            $str->years = get_string('years');
         }
 
         ob_start();
-        echo '<html><head>';
-        echo '<script language="JavaScript">';
-        echo "<!-- //hide\n";
-
-        echo 'function openpopup(url,name,options,fullscreen) {';
-        echo 'fullurl = "'.$CFG->wwwroot.'" + url;';
-        echo 'windowobj = window.open(fullurl,name,options);';
-        echo 'if (fullscreen) {';
-        echo '  windowobj.moveTo(0,0);';
-        echo '  windowobj.resizeTo(screen.availWidth,screen.availHeight); ';
-        echo '}';
-        echo 'windowobj.focus();';
-        echo 'return false;';
-        echo "}\n-->\n";
-        echo '</script></head><body style="font-face: serif;" bgcolor="#FFFFFF">';
-
-        echo '<table style="width: 100%;"><tbody>';
+        $refresh_inval = $CFG->chat_refresh_userlist * 1000;
+        echo <<<EOD
+        <html><head>
+        <meta http-equiv="refresh" content="$refresh_inval">
+        <style type="text/css"> img{border:0} </style>
+        <script type="text/javascript">
+        //<![CDATA[
+        function openpopup(url,name,options,fullscreen) {
+            fullurl = "$CFG->wwwroot" + url;
+            windowobj = window.open(fullurl,name,options);
+            if (fullscreen) {
+                windowobj.moveTo(0,0);
+                windowobj.resizeTo(screen.availWidth,screen.availHeight);
+            }
+            windowobj.focus();
+            return false;
+        }
+        //]]>
+        </script></head><body><table><tbody>
+EOD;
 
         // Get the users from that chatroom
         $users = $this->chatrooms[$info['chatid']]['users'];
@@ -495,6 +506,7 @@ class ChatDaemon {
         $this->sets_info[$sessionid] = array(
             'lastinfocommit' => 0,
             'lastmessageindex' => 0,
+            'course'    => $course,
             'courseid'  => $course->id,
             'chatuser'  => $chatuser,
             'chatid'    => $chat->id,
@@ -700,7 +712,7 @@ class ChatDaemon {
             {
 
                 // Simply give them the message
-                chat_force_language($info['lang']);
+                course_setup($info['course'], $info['user']);
                 $output = chat_format_message_manually($message, $info['courseid'], $sender, $info['user']);
                 $this->trace('Delivering message "'.$output->text.'" to '.$this->conn_sets[$sessionid][CHAT_CONNECTION_CHANNEL]);
 
