@@ -1,4 +1,4 @@
-<?php  // $Id: rsslib.php,v 1.15.2.2 2006/06/07 08:16:01 vyshane Exp $
+<?php  // $Id: rsslib.php,v 1.25.2.1 2008/07/10 09:48:46 scyrma Exp $
     //This file adds support to rss feeds generation
 
     //This function is the main entry point to forum
@@ -12,16 +12,10 @@
 
         //Check CFG->enablerssfeeds
         if (empty($CFG->enablerssfeeds)) {
-            //Some debug...
-            if ($CFG->debug > 7) {
-                echo "DISABLED (admin variables)";
-            }
+            debugging('DISABLED (admin variables)');
         //Check CFG->forum_enablerssfeeds
         } else if (empty($CFG->forum_enablerssfeeds)) {
-            //Some debug...
-            if ($CFG->debug > 7) {
-                echo "DISABLED (module configuration)";
-            }
+            debugging('DISABLED (module configuration)');
         //It's working so we start...
         } else {
             //Iterate over all forums
@@ -50,25 +44,20 @@
 
                         mtrace("Updating RSS feed for ".format_string($forum->name,true).", ID: $forum->id");
 
-                        //Some debug...
-                        if ($CFG->debug > 7) {
-                            echo "ID: $forum->id->";
-                        }
                         //Get the XML contents
                         $result = forum_rss_feed($forum);
                         //Save the XML contents to file
                         if (!empty($result)) {
                             $status = rss_save_file("forum",$forum,$result);
                         }
-                        //Some debug...
-                        if ($CFG->debug > 7) {
+                        if (debugging()) {
                             if (empty($result)) {
-                                echo "(empty) ";
+                                echo "ID: $forum->id-> (empty) ";
                             } else {
                                 if (!empty($status)) {
-                                    echo "OK ";
+                                    echo "ID: $forum->id-> OK ";
                                 } else {
-                                    echo "FAIL ";
+                                    echo "ID: $forum->id-> FAIL ";
                                 }
                             }
                         }
@@ -78,16 +67,19 @@
         }
         return $status;
     }
-    
-    
+
+
     // Given a forum object, deletes the RSS file
     function forum_rss_delete_file($forum) {
         global $CFG;
-        //return unlink("{$CFG->dataroot}/rss/{$modname}/{$forum->id}.xml");
         $rssfile = rss_file_name('forum', $forum);
-        return unlink($rssfile);
+        if (file_exists($rssfile)) {
+            return unlink($rssfile);
+        } else {
+            return true;
+        }
     }
-    
+
 
     function forum_rss_newstuff($forum, $time) {
     // If there is new stuff in the forum since $time then this returns
@@ -110,16 +102,10 @@
 
         //Check CFG->enablerssfeeds
         if (empty($CFG->enablerssfeeds)) {
-            //Some debug...
-            if ($CFG->debug > 7) {
-                echo "DISABLED (admin variables)";
-            }
+            debugging("DISABLED (admin variables)");
         //Check CFG->forum_enablerssfeeds
         } else if (empty($CFG->forum_enablerssfeeds)) {
-            //Some debug...
-            if ($CFG->debug > 7) {
-                echo "DISABLED (module configuration)";
-            }
+            debugging("DISABLED (module configuration)");
         //It's working so we start...
         } else {
             //Check the forum has rss activated
@@ -168,7 +154,7 @@
         $items = array();
 
         if ($newsince) {
-            $newsince = " AND (p.modified > '$newsince' OR d.timemodified > '$newsince')";
+            $newsince = " AND p.modified > '$newsince'";
         } else {
             $newsince = "";
         }
@@ -188,32 +174,25 @@
                                             p.discussion = d.id AND
                                             p.parent = 0 AND
                                             u.id = p.userid $newsince
-                                      ORDER BY p.created desc")) {
+                                      ORDER BY p.created desc", 0, $forum->rssarticles)) {
 
-            //Are we just looking for new ones?  If so, then return now.
-            if ($newsince) {
-                return true;
-            }
-
-            //Iterate over each discussion to get forum->rssarticles records
-            $articlesleft = $forum->rssarticles;
             $item = NULL;
             $user = NULL;
+
+            $formatoptions = new object;
+            $formatoptions->trusttext = true;
+
             foreach ($recs as $rec) {
                 unset($item);
                 unset($user);
-                $item->title = $rec->discussionname;
+                $item->title = format_string($rec->discussionname);
                 $user->firstname = $rec->userfirstname;
                 $user->lastname = $rec->userlastname;
                 $item->author = fullname($user);
                 $item->pubdate = $rec->postcreated;
                 $item->link = $CFG->wwwroot."/mod/forum/discuss.php?d=".$rec->discussionid;
-                $item->description = format_text($rec->postmessage,$rec->postformat,NULL,$forum->course);
+                $item->description = format_text($rec->postmessage,$rec->postformat,$formatoptions,$forum->course);
                 $items[] = $item;
-                $articlesleft--;
-                if ($articlesleft < 1) {
-                    break;
-                }
             }
         }
         return $items;
@@ -228,7 +207,7 @@
         $items = array();
 
         if ($newsince) {
-            $newsince = " AND (p.modified > '$newsince' OR d.timemodified > '$newsince')";
+            $newsince = " AND p.modified > '$newsince'";
         } else {
             $newsince = "";
         }
@@ -249,17 +228,16 @@
                                       WHERE d.forum = '$forum->id' AND
                                             p.discussion = d.id AND
                                             u.id = p.userid $newsince
-                                      ORDER BY p.created desc")) {
+                                      ORDER BY p.created desc", 0, $forum->rssarticles)) {
 
-            //Are we just looking for new ones?  If so, then return now.
-            if ($newsince) {
-                return true;
-            }
-
-            //Iterate over each discussion to get forum->rssarticles records
-            $articlesleft = $forum->rssarticles;
             $item = NULL;
             $user = NULL;
+
+            $formatoptions = new object;
+            $formatoptions->trusttext = true;
+
+            require_once($CFG->libdir.'/filelib.php');
+
             foreach ($recs as $rec) {
                 unset($item);
                 unset($user);
@@ -270,31 +248,23 @@
                 $item->author = fullname($user);
                 $item->pubdate = $rec->postcreated;
                 $item->link = $CFG->wwwroot."/mod/forum/discuss.php?d=".$rec->discussionid."&parent=".$rec->postid;
-                $item->description = format_text($rec->postmessage,$rec->postformat,NULL,$forum->course);
+                $item->description = format_text($rec->postmessage,$rec->postformat,$formatoptions,$forum->course);
 
 
-                $post_file_area_name = "$forum->course/$CFG->moddata/forum/$forum->id/$rec->postid";
+                $post_file_area_name = str_replace('//', '/', "$forum->course/$CFG->moddata/forum/$forum->id/$rec->postid");
                 $post_files = get_directory_list("$CFG->dataroot/$post_file_area_name");
                 
                 if (!empty($post_files)) {            
                     $item->attachments = array();
                     foreach ($post_files as $file) {                    
                         $attachment = new stdClass;
-                        if ($CFG->slasharguments) {
-                            $attachment->url = "{$CFG->wwwroot}/file.php/$post_file_area_name/$file";
-                        } else {
-                            $attachment->url = "{$CFG->wwwroot}/file.php?file=/$post_file_area_name/$file";
-                        }                         
+                        $attachment->url = get_file_url("$post_file_area_name/$file");
                         $attachment->length = filesize("$CFG->dataroot/$post_file_area_name/$file");
                         $item->attachments[] = $attachment;
                     }
                 }
 
                 $items[] = $item;
-                $articlesleft--;
-                if ($articlesleft < 1) {
-                    break;
-                }
             }
         }
         return $items;
