@@ -1,4 +1,4 @@
-<?php   // $Id: print.php,v 1.32.2.1 2006/05/07 10:48:10 stronk7 Exp $
+<?php   // $Id: print.php,v 1.41.2.2 2009/01/16 04:55:38 dongsheng Exp $
 
     global $CFG;
 
@@ -14,7 +14,7 @@
     $hook    = optional_param('hook','ALL', PARAM_ALPHANUM);   // what to show
     $sortkey = optional_param('sortkey','UPDATE', PARAM_ALPHA); // Sorting key
 
-    if (! $cm = get_record("course_modules", "id", $id)) {
+    if (! $cm = get_coursemodule_from_id('glossary', $id)) {
         error("Course Module ID was incorrect");
     }
 
@@ -33,11 +33,12 @@
     print_header();
 
     require_course_login($course, true, $cm);
+    $context = get_context_instance(CONTEXT_MODULE, $cm->id);
 
 /// Loading the textlib singleton instance. We are going to need it.
     $textlib = textlib_get_instance();
 
-    if (!isteacher($course->id) and !$glossary->allowprintview) {
+    if (!has_capability('mod/glossary:manageentries', $context) and !$glossary->allowprintview) {
         notice(get_string('printviewnotallowed', 'glossary'));
     }
 
@@ -143,19 +144,19 @@
     }
 
     $site = get_record("course","id",1);
-    echo '<p align="right"><font size="-1">' . userdate(time()) . '</font></p>';
-    echo get_string("site") . ': <strong>' . $site->fullname . '</strong><br />';
-    echo get_string("course") . ': <strong>' . $course->fullname . ' ('. $course->shortname . ')</strong><br />';
+    echo '<p style="text-align:right"><span style="font-size:0.75em">' . userdate(time()) . '</span></p>';
+    echo get_string("site") . ': <strong>' . format_string($site->fullname) . '</strong><br />';
+    echo get_string("course") . ': <strong>' . format_string($course->fullname) . ' ('. format_string($course->shortname) . ')</strong><br />';
     echo get_string("modulename","glossary") . ': <strong>' . format_string($glossary->name, true) . '</strong>';
     if ( $allentries ) {
         foreach ($allentries as $entry) {
 
             // Setting the pivot for the current entry
-            $pivot = $entry->pivot;
-            $upperpivot = $textlib->strtoupper($pivot, current_charset());
+            $pivot = $entry->glossarypivot;
+            $upperpivot = $textlib->strtoupper($pivot);
             // Reduce pivot to 1cc if necessary
             if ( !$fullpivot ) {
-                $upperpivot = $textlib->substr($upperpivot, 0, 1, current_charset());
+                $upperpivot = $textlib->substr($upperpivot, 0, 1);
             }            
             
             // If there's  group break
@@ -166,13 +167,13 @@
                     $currentpivot = $upperpivot;
 
                     $pivottoshow = $currentpivot;
-                    if ( isset($entry->uid) ) {
+                    if ( isset($entry->userispivot) ) {
                         // printing the user icon if defined (only when browsing authors)
-                        $user = get_record("user","id",$entry->uid);
-                        $pivottoshow = fullname($user, isteacher($course->id));
+                        $user = get_record("user","id",$entry->userid);
+                        $pivottoshow = fullname($user);
                     }
 
-                    echo "<p align=\"center\"><strong>".clean_text($pivottoshow)."</strong></p>" ;
+                    echo "<p class='mdl-align'><strong>".clean_text($pivottoshow)."</strong></p>" ;
                 }
             }
 
@@ -180,5 +181,5 @@
         }
     }
 
-    echo '</div></div></body></html>';
+    print_footer('empty');
 ?>
