@@ -1,4 +1,4 @@
-<?php //$Id: restorelib.php,v 1.45 2006/04/11 16:13:50 stronk7 Exp $
+<?php //$Id: restorelib.php,v 1.53 2006/11/15 03:15:59 toyomoyo Exp $
     //This php script contains all the stuff to backup/restore
     //glossary mods
 
@@ -41,6 +41,10 @@
         if ($data) {
             //Now get completed xmlized object
             $info = $data->info;
+            // if necessary, write to restorelog and adjust date/time fields
+            if ($restore->course_startdateoffset) {
+                restore_log_date_changes('Glossary', $restore, $info['MOD']['#'], array('ASSESSTIMESTART', 'ASSESSTIMEFINISH'));
+            }
             //traverse_xmlize($info);                                                                     //Debug
             //print_object ($GLOBALS['traverse_array']);                                                  //Debug
             //$GLOBALS['traverse_array']="";                                                              //Debug
@@ -49,7 +53,6 @@
             $glossary->course = $restore->course_id;
             $glossary->name = backup_todb($info['MOD']['#']['NAME']['0']['#']);
             $glossary->intro = backup_todb($info['MOD']['#']['INTRO']['0']['#']);
-            $glossary->studentcanpost = backup_todb($info['MOD']['#']['STUDENTCANPOST']['0']['#']);
             $glossary->allowduplicatedentries = backup_todb($info['MOD']['#']['ALLOWDUPLICATEDENTRIES']['0']['#']);
             $glossary->displayformat = backup_todb($info['MOD']['#']['DISPLAYFORMAT']['0']['#']);
             $glossary->mainglossary = backup_todb($info['MOD']['#']['MAINGLOSSARY']['0']['#']);
@@ -144,7 +147,7 @@
         $status = true;
 
         //Get the entries array
-        $entries = $info['MOD']['#']['ENTRIES']['0']['#']['ENTRY'];
+        $entries = isset($info['MOD']['#']['ENTRIES']['0']['#']['ENTRY'])?$info['MOD']['#']['ENTRIES']['0']['#']['ENTRY']:array();
 
         //Iterate over entries
         for($i = 0; $i < sizeof($entries); $i++) {
@@ -229,7 +232,7 @@
         $status = true;
 
         //Get the comments array
-        $comments = $info['#']['COMMENTS']['0']['#']['COMMENT'];
+        $comments = isset($info['#']['COMMENTS']['0']['#']['COMMENT'])?$info['#']['COMMENTS']['0']['#']['COMMENT']:array();
 
         //Iterate over comments
         for($i = 0; $i < sizeof($comments); $i++) {
@@ -244,7 +247,11 @@
             //Now, build the GLOSSARY_COMMENTS record structure
             $comment->entryid = $new_entry_id;
             $comment->userid = backup_todb($com_info['#']['USERID']['0']['#']);
-            $comment->comment = backup_todb($com_info['#']['COMMENT']['0']['#']);
+            if (isset($com_info['#']['COMMENT']['0']['#'])) {
+                $comment->entrycomment = backup_todb($com_info['#']['COMMENT']['0']['#']);
+            } else {
+                $comment->entrycomment = backup_todb($com_info['#']['ENTRYCOMMENT']['0']['#']);
+            }
             $comment->timemodified = backup_todb($com_info['#']['TIMEMODIFIED']['0']['#']);
             $comment->format = backup_todb($com_info['#']['FORMAT']['0']['#']);
 
@@ -286,7 +293,7 @@
         $status = true;
 
         //Get the ratings array
-        $ratings = $info['#']['RATINGS']['0']['#']['RATING'];
+        $ratings = isset($info['#']['RATINGS']['0']['#']['RATING'])?$info['#']['RATINGS']['0']['#']['RATING']:array();
 
         //Iterate over ratings
         for($i = 0; $i < sizeof($ratings); $i++) {
@@ -337,7 +344,7 @@
         $status = true;
 
         //Get the comments array
-        $aliases = $info['#']['ALIASES']['0']['#']['ALIAS'];
+        $aliases = isset($info['#']['ALIASES']['0']['#']['ALIAS'])?$info['#']['ALIASES']['0']['#']['ALIAS']:array();
 
         //Iterate over comments
         for($i = 0; $i < sizeof($aliases); $i++) {
@@ -377,7 +384,7 @@
         $status = true;
 
         //Get the categories array
-        $categories = $info['MOD']['#']['CATEGORIES']['0']['#']['CATEGORY'];
+        $categories = isset($info['MOD']['#']['CATEGORIES']['0']['#']['CATEGORY'])?$info['MOD']['#']['CATEGORIES']['0']['#']['CATEGORY']:array();
 
         //Iterate over categories
         for($i = 0; $i < sizeof($categories); $i++) {
@@ -429,7 +436,7 @@
         $status = true;
 
         //Get the entryids array
-        $entryids = $info['#']['ENTRIES']['0']['#']['ENTRY'];
+        $entryids = isset($info['#']['ENTRIES']['0']['#']['ENTRY'])?$info['#']['ENTRIES']['0']['#']['ENTRY']:array();
 
         //Iterate over entryids
         for($i = 0; $i < sizeof($entryids); $i++) {
@@ -612,7 +619,7 @@
                     //Update record
                     $entry->definition = addslashes($result);
                     $status = update_record("glossary_entries",$entry);
-                    if ($CFG->debug>7) {
+                    if (debugging()) {
                         if (!defined('RESTORE_SILENTLY')) {
                             echo '<br /><hr />'.s($content).'<br />changed to<br />'.s($result).'<hr /><br />';
                         }
@@ -646,7 +653,7 @@
                     //Update record
                     $glossary->intro = addslashes($result);
                     $status = update_record("glossary",$glossary);
-                    if ($CFG->debug>7) {
+                    if (debugging()) {
                         if (!defined('RESTORE_SILENTLY')) {
                             echo '<br /><hr />'.s($content).'<br />changed to<br />'.s($result).'<hr /><br />';
                         }
@@ -676,8 +683,8 @@
 
         $status = true;
 
-        //Convert glossary_comments->comment
-        if ($records = get_records_sql ("SELECT c.id, c.comment, c.format
+        //Convert glossary_comments->entrycomment
+        if ($records = get_records_sql ("SELECT c.id, c.entrycomment, c.format
                                          FROM {$CFG->prefix}glossary_comments c,
                                               {$CFG->prefix}glossary_entries e,
                                               {$CFG->prefix}glossary g,
@@ -691,10 +698,10 @@
                                                b.new_id = c.id")) {
             foreach ($records as $record) {
                 //Rebuild wiki links
-                $record->comment = restore_decode_wiki_content($record->comment, $restore);
+                $record->entrycomment = restore_decode_wiki_content($record->entrycomment, $restore);
                 //Convert to Markdown
                 $wtm = new WikiToMarkdown();
-                $record->comment = $wtm->convert($record->comment, $restore->course_id);
+                $record->entrycomment = $wtm->convert($record->entrycomment, $restore->course_id);
                 $record->format = FORMAT_MARKDOWN;
                 $status = update_record('glossary_comments', addslashes_object($record));
                 //Do some output
