@@ -1,17 +1,17 @@
-<?PHP //$Id: restorelib.php,v 1.11 2006/01/25 07:20:13 gbateson Exp $
+<?PHP //$Id: restorelib.php,v 1.24.4.5 2009/09/25 06:21:45 gbateson Exp $
 //This php script contains all the stuff to restore hotpot mods
     //-----------------------------------------------------------
     // This is the "graphical" structure of the hotpot mod:
     //-----------------------------------------------------------
     //
     //                         hotpot
-    //                      (CL, pk->id, 
+    //                      (CL, pk->id,
     //                   fk->course, files)
     //                           |
     //            +--------------+---------------+
     //            |                              |
     //      hotpot_attempts             hotpot_questions
-    //       (UL, pk->id,                 (UL, pk->id, 
+    //       (UL, pk->id,                 (UL, pk->id,
     //        fk->hotpot)               fk->hotpot, text)
     //            |                              |    |
     //            +-------------------+----------+    |
@@ -40,7 +40,7 @@ require_once ("$CFG->dirroot/mod/hotpot/lib.php");
 function hotpot_restore_mods($mod, $restore) {
     //This function restores a single hotpot activity
 
-    // This function is called by "restore_create_modules" (in "backup/restorelib.php") 
+    // This function is called by "restore_create_modules" (in "backup/restorelib.php")
     // which is called by "backup/restore_execute.html" (included by "backup/restore.php")
     // $mod is an object
     //     id           : id field in 'modtype' table
@@ -49,17 +49,18 @@ function hotpot_restore_mods($mod, $restore) {
     //     backup_unique_code : xxxxxxxxxx
     //     file         : '/full/path/to/backupfile.zip'
     //     mods         : an array of $modinfo's (see below)
-    //     restoreto    : 0=existing course (replace), 1=existing course (append), 2=new course
+    //     restoreto    : See RESTORETO_XXX constants in backup/lib.php
     //     users        : 0=all, 1=course, 2=none
     //     logs         : 0=no, 1=yes
     //     user_files   : 0=no, 1=yes
     //     course_files : 0=no, 1=yes
     //     course_id    : id of course into which data is to be restored
-    //     deleting     : true if 'restoreto'==0, otherwise false
+    //     deleting     : true if 'restoreto'==RESTORETO_NEW_COURSE, otherwise false
     //     original_wwwroot : 'http://your.server.com/moodle'
     // $modinfo is an array
     //    'modname'    : array( 'restore'=> 0=no 1=yes, 'userinfo' => 0=no 1=yes)
 
+    global $CFG;
     $status = true;
 
     // get course module data this hotpot activity
@@ -77,7 +78,7 @@ function hotpot_restore_mods($mod, $restore) {
         $more_restore = '';
         // print a message after each hotpot is backed up
         if (!defined('RESTORE_SILENTLY')) {
-            $more_restore .= 'print "<li>".get_string("modulename", "hotpot")." &quot;".$record->name."&quot;</li>";';
+            $more_restore .= 'print "<li>".get_string("modulename", "hotpot")." &quot;".format_string(stripslashes($record->name),true)."&quot;</li>";';
         }
         $more_restore .= 'backup_flush(300);';
         if (function_exists('restore_userdata_selected')) {
@@ -88,16 +89,29 @@ function hotpot_restore_mods($mod, $restore) {
             $restore_userdata_selected = $restore->mods['hotpot']->userinfo;
         }
         if ($restore_userdata_selected) {
-            if (isset($xml["STRING_DATA"]) && isset($xml["QUESTION_DATA"])) {
+            $has_details = false;
+            if (isset($xml["ATTEMPT_DATA"]["0"]["#"]["ATTEMPT"]["0"]["#"]["DETAILS"]["0"]["#"])) {
+                $details = trim($xml["ATTEMPT_DATA"]["0"]["#"]["ATTEMPT"]["0"]["#"]["DETAILS"]["0"]["#"]);
+                if ($details<>'' && $details<>'<?xml version="1.0"?><hpjsresult><fields></fields></hpjsresult>') {
+                    $has_details = true;
+                }
+            }
+            if ($has_details && empty($xml["STRING_DATA"]) && empty($xml["QUESTION_DATA"])) {
+                // HotPot v2.0.x (regenerate questions, responses and strings from attempt details)
+                $more_restore .= '$status = hotpot_restore_attempts($restore, $status, $xml, $record, true);';
+            } else {
                 // HotPot v2.1+
                 $more_restore .= '$status = hotpot_restore_strings($restore, $status, $xml, $record);';
                 $more_restore .= '$status = hotpot_restore_questions($restore, $status, $xml, $record);';
                 $more_restore .= '$status = hotpot_restore_attempts($restore, $status, $xml, $record);';
-            } else {
-                // HotPot v2.0.x (regenerate questions, responses and strings from attempt details)
-                $more_restore .= '$status = hotpot_restore_attempts($restore, $status, $xml, $record, true);';
             }
         }
+
+        // if necessary, adjust HotPot date/time fields and write to restorelog
+        if ($restore->course_startdateoffset) {
+            restore_log_date_changes('Hotpot', $restore, $xml, array('TIMEOPEN', 'TIMECLOSE', 'TIMECREATED', 'TIMEMODIFIED'));
+        }
+
         $status = hotpot_restore_records(
             $restore, $status, $xml, $table, $foreign_keys, $more_restore
         );
@@ -108,14 +122,14 @@ function hotpot_restore_strings(&$restore, $status, &$xml, &$record) {
     // $xml is an XML tree for a hotpot record
     // $record is the newly added hotpot record
     return hotpot_restore_records(
-        $restore, $status, $xml, 'hotpot_strings', array(), '', 'STRING_DATA', 'STRING', 'string'
+        $restore, $status, $xml, 'hotpot_strings', array(), '', 'STRING_DATA', 'STRING', 'md5key'
     );
 }
 function hotpot_restore_questions(&$restore, $status, &$xml, &$record) {
     // $xml is an XML tree for a hotpot record
     // $record is the newly added hotpot record
     $foreignkeys = array(
-        'hotpot'=>$record->id, 
+        'hotpot'=>$record->id,
         'text'=>'hotpot_strings'
     );
     return hotpot_restore_records(
@@ -203,7 +217,7 @@ function hotpot_restore_details(&$restore, $status, &$xml, &$record) {
     if (empty($record->details)) {
         $status = true;
     } else {
-        unset($details);
+        $details = new stdClass();
         $details->attempt = $record->id;
         $details->details = $record->details;
         if (insert_record('hotpot_details', $details)) {
@@ -226,7 +240,8 @@ function hotpot_restore_records(&$restore, $status, &$xml, $table, $foreign_keys
     // $record_TAG  : (optional) the name of an XML tag which starts a single record
     //    If no $record_TAG is specified, the block of records is assumed to be a single record
     // other parameters are explained in "hotpot_restore_record" below
-    $i = 0;
+
+    $i = 0; // index for $records_TAG
     do {
         unset($xml_records);
         if ($records_TAG) {
@@ -238,10 +253,8 @@ function hotpot_restore_records(&$restore, $status, &$xml, $table, $foreign_keys
                 $xml_records = &$xml;
             }
         }
-        if (empty($xml_records)) {
-            // do nothing
-        } else {
-            $ii = 0;
+        if (isset($xml_records)) {
+            $ii = 0; // index for $record_TAG
             do {
                 unset($xml_record);
                 if ($record_TAG) {
@@ -253,9 +266,7 @@ function hotpot_restore_records(&$restore, $status, &$xml, $table, $foreign_keys
                         $xml_record = &$xml_records;
                     }
                 }
-                if (empty($xml_record)) {
-                    // do nothing
-                } else {
+                if (isset($xml_record)) {
                     $status = hotpot_restore_record(
                         $restore, $status, $xml_record, $table, $foreign_keys, $more_restore, $secondary_key
                     );
@@ -276,14 +287,23 @@ function hotpot_restore_record(&$restore, $status, &$xml, $table, $foreign_keys,
     // $foreign_keys : array of foreign keys, if any, specifed as $key=>$value
     //    $key   : the name of a field in the current $record
     //    $value : if $value is numeric, then $record->$key is set to $value.
-    //        Otherwise $value is assumed to be a table name and $record->$key 
+    //        Otherwise $value is assumed to be a table name and $record->$key
     //        is treated as a comma separated list of ids in that table
     // $more_restore : optional PHP code to be eval(uated) for each record
-    // $secondary_key : 
+    // $secondary_key :
     //    the name of the secondary key field, if any, in the current $record.
     //    If this field is specified, then the current record will only be added
     //    if the $record->$secondarykey value does not already exist in $table
-    unset($record);
+
+    // maintain a cache of info on table columns
+    static $table_columns = array();
+    if (empty($table_columns[$table])) {
+        global $CFG, $db;
+        $table_columns[$table] = $db->MetaColumns("$CFG->prefix$table");
+    }
+
+    // get values for fields in this record
+    $record = new stdClass();
     $TAGS = array_keys($xml);
     foreach ($TAGS as $TAG) {
         $value = $xml[$TAG][0]['#'];
@@ -292,41 +312,70 @@ function hotpot_restore_record(&$restore, $status, &$xml, $table, $foreign_keys,
             $record->$tag = backup_todb($value);
         }
     }
+
+    // update foreign keys, if any
     $ok = true;
     foreach ($foreign_keys as $key=>$value) {
         if (is_numeric($value)) {
             $record->$key = $value;
         } else {
-            if (empty($record->$key)) {
-                $record->$key = NULL;
-            } else {
-                $key_table = $value;
-                $new_ids = array();
+            $key_table = $value;
+            $new_ids = array();
+            if (isset($record->$key)) {
                 $old_ids = explode(',', $record->$key);
                 foreach ($old_ids as $old_id) {
-                    $key_record = backup_getid($restore->backup_unique_code, $key_table, $old_id);
-                    if ($key_record) {
-                        $new_ids[] = $key_record->new_id;
+                    if (empty($old_id)) {
+                        // do nothing
                     } else {
-                        // foreign key could not be updated
-                        if (!defined('RESTORE_SILENTLY')) {
-                            print "<ul><li><b>Warning:</b><br>Foreign key could not be updated:<br>";
-                            print "'$key_table' record (old id=$old_id) is missing from backup data<br>";
-                            print "'$table' record ";
-                            if (isset($record->id)) {
-                                print "(old id=$record->id) ";
+                        $key_record = backup_getid($restore->backup_unique_code, $key_table, $old_id);
+                        if ($key_record) {
+                            $new_ids[] = $key_record->new_id;
+                        } else {
+                            // foreign key could not be updated
+                            if (!defined('RESTORE_SILENTLY')) {
+                                print "<ul><li><b>Warning:</b><br/>Foreign key could not be updated:<br/>";
+                                print "'$key_table' record (old id=$old_id) is missing from backup data<br/>";
+                                print "'$table' record ";
+                                if (isset($record->id)) {
+                                    print "(old id=$record->id) ";
+                                }
+                                print "was not restored</li></ul>";
                             }
-                            print "was not restored</li></ul>";
+                            $ok = false;
                         }
-                        $ok = false; 
                     }
                 }
-                $record->$key = implode(',', $new_ids);
+            }
+            $record->$key = implode(',', $new_ids);
+        }
+    }
+
+    // set md5 keys if necessary (restoring from Moodle<1.6)
+    if ($table=='hotpot_questions' && empty($record->md5key)) {
+        $record->md5key = md5($record->name);
+    }
+    if ($table=='hotpot_strings' && empty($record->md5key)) {
+        $record->md5key = md5($record->string);
+    }
+
+    // check all "not null" fields have been set
+    foreach ($table_columns[$table] as $column) {
+        if ($column->not_null) {
+            $name = $column->name;
+            if ($name=='id' || (isset($record->$name) && ! is_null($record->$name))) {
+                // do nothing
+            } else if (isset($column->default_value)) {
+                $record->$name = $column->default_value;
+            } else if (preg_match('/int|decimal|double|float|time|year/i', $column->type)) {
+                $record->$name = 0;
+            } else {
+                $record->$name = '';
             }
         }
     }
+
     // check everything is OK so far
-    if ($ok && isset($record)) {
+    if ($ok) {
         // store old record id, if necessary
         if (isset($record->id)) {
             $record->old_id = $record->id;
@@ -335,15 +384,16 @@ function hotpot_restore_record(&$restore, $status, &$xml, $table, $foreign_keys,
         // if there is a secondary key field  ...
         if ($secondary_key) {
             // check to see if a record with the same value already exists
-            $key_record = get_record($table, $secondary_key, $record->$secondary_key);
-            if ($key_record) {
+            $key_records = get_records($table, $secondary_key, $record->$secondary_key);
+            if ($key_records) {
                 // set new record id from already existing record
+                $key_record = reset($key_records);
                 $record->id = $key_record->id;
             }
         }
         if (empty($record->id)) {
             // add the $record (and get new id)
-            $record->id = insert_record ($table, $record);
+            $record->id = insert_record($table, $record);
         }
         // check $record was added (or found)
         if (is_numeric($record->id)) {
@@ -351,7 +401,7 @@ function hotpot_restore_record(&$restore, $status, &$xml, $table, $foreign_keys,
             if (isset($record->old_id)) {
                 backup_putid($restore->backup_unique_code, $table, $record->old_id, $record->id);
             }
-        } else { 
+        } else {
             // failed to add (or find) $record
             if (!defined('RESTORE_SILENTLY')) {
                 print "<ul><li>Record could not be added: table=$table</li></ul>";
@@ -401,7 +451,7 @@ function hotpot_restore_logs($restore, $log) {
         break;
         case "attempt":
         case "submit":
-        case "review": 
+        case "review":
             if ($log->cmid) {
                 //Get the new_id of the module (to recode the info field)
                 $mod = backup_getid($restore->backup_unique_code,$log->module,$log->info);
@@ -410,7 +460,7 @@ function hotpot_restore_logs($restore, $log) {
                     $attemptid = substr(strrchr($log->url,"="),1);
                     //Get the new_id of the attempt (to recode the url field)
                     $attempt = backup_getid($restore->backup_unique_code,"hotpot_attempts",$attemptid);
-                    if ($attempt) { 
+                    if ($attempt) {
                         $log->url = "review.php?id=".$log->cmid."&attempt=".$attempt->new_id;
                         $log->info = $mod->new_id;
                         $status = true;
@@ -427,4 +477,46 @@ function hotpot_restore_logs($restore, $log) {
     } // end switch
     return $status ? $log : false;
 }
+
+function hotpot_decode_content_links($content, $restore) {
+    $search = '/\$@(HOTPOT)\*([a-z]+)\*([a-z]+)\*([0-9]+)@\$/ise';
+    $replace = 'hotpot_decode_content_link("$2", "$3", "$4", $restore)';
+    return preg_replace($search, $replace, $content);
+}
+
+function hotpot_decode_content_link($scriptname, $paramname, $paramvalue, &$restore) {
+    global $CFG;
+
+    $table = '';
+    switch ($paramname) {
+        case 'id':
+            switch ($scriptname) {
+                case 'index':
+                    $table = 'course';
+                    break;
+                case 'report':
+                case 'review':
+                case 'view':
+                    $table = 'course_modules';
+                    break;
+                case 'attempt':
+                    $table = 'hotpot_attempts';
+                    break;
+            }
+            break;
+        case 'hp':
+        case 'hotpotid':
+            $table = 'hotpot';
+            break;
+    }
+
+    $new_id = 0;
+    if ($table) {
+        if ($rec = backup_getid($restore->backup_unique_code, $table, $paramvalue)) {
+            $new_id = $rec->new_id;
+        }
+    }
+
+    return "$CFG->wwwroot/mod/hotpot/$scriptname.php?$paramname=$new_id";
+}
 ?>
