@@ -1,5 +1,10 @@
 <?php
 
+// THIS FILE IS DEPRECATED!  PLEASE DO NOT MAKE CHANGES TO IT!
+//
+// IT IS USED ONLY FOR UPGRADES FROM BEFORE MOODLE 1.7, ALL 
+// LATER CHANGES SHOULD USE upgrade.php IN THIS DIRECTORY.
+
 function label_upgrade($oldversion) {
 
 /// This function does anything necessary to upgrade 
@@ -22,9 +27,9 @@ function label_upgrade($oldversion) {
 
     if ($oldversion < 2004021900) {
 
-        modify_database("", "INSERT INTO prefix_log_display (module, action, mtable, field) VALUES ('label', 'add', 'quiz', 'name');");
+        modify_database("", "INSERT INTO prefix_log_display (module, action, mtable, field) VALUES ('label', 'add', 'label', 'name');");
 
-        modify_database("", "INSERT INTO prefix_log_display (module, action, mtable, field) VALUES ('label', 'update', 'quiz', 'name');");
+        modify_database("", "INSERT INTO prefix_log_display (module, action, mtable, field) VALUES ('label', 'update', 'label', 'name');");
 
     }
 
@@ -34,6 +39,8 @@ function label_upgrade($oldversion) {
         modify_database('','CREATE INDEX prefix_label_course_idx ON prefix_label (course);');
     }
 
+    //////  DO NOT ADD NEW THINGS HERE!!  USE upgrade.php and the lib/ddllib.php functions.
+
     return true;
 
 }
