@@ -1,12 +1,12 @@
-<?php
-
-/******************* confirm delete ************************************/
-
-    if (!isteacher($course->id)) {
-        error("Only teachers can look at this page");
-    }
-
-    confirm_sesskey();
+<?php // $Id: confirmdelete.php,v 1.6.2.1 2009/11/02 17:01:07 tjhunt Exp $
+/**
+ * Action for confirming the deletion of a page
+ *
+ * @version $Id: confirmdelete.php,v 1.6.2.1 2009/11/02 17:01:07 tjhunt Exp $
+ * @license http://www.gnu.org/copyleft/gpl.html GNU Public License
+ * @package lesson
+ **/
+    require_sesskey();
 
     $pageid = required_param('pageid', PARAM_INT);
     if (!$thispage = get_record("lesson_pages", "id", $pageid)) {
@@ -28,4 +28,3 @@
          "lesson.php?action=delete&amp;id=$cm->id&amp;pageid=$pageid&amp;sesskey=".$USER->sesskey, 
          "view.php?id=$cm->id");
 ?>
-    
