@@ -1,9 +1,9 @@
-<?php    // $Id: localfile.php,v 1.1 2005/04/02 12:11:27 moodler Exp $
+<?php    // $Id: localfile.php,v 1.6.4.3 2008/12/10 06:30:25 dongsheng Exp $
 
     require('../../../../config.php');
     require('../../lib.php');
 
-    $choose = required_param('choose');
+    $choose = required_param('choose', PARAM_FILE);
 
     require_login();
 
@@ -15,11 +15,9 @@
 
     print_simple_box(get_string('localfileinfo', 'resource'), 'center');
 
-    $chooseparts = explode('.', $choose);
-
     ?>
-    <script language="javascript" type="text/javascript">
-    <!--
+    <script type="text/javascript">
+    //<![CDATA[
     function set_value(txt) {
         if (txt.indexOf('/') > -1) {
             path = txt.substring(txt.indexOf('/'),txt.length);
@@ -28,22 +26,25 @@
         } else {
             window.close();
         }
-        opener.document.forms['<?php echo $chooseparts[0]."'].".$chooseparts[1] ?>.value = '<?php p(RESOURCE_LOCALPATH) ?>'+path;
+        opener.document.getElementById('<?php echo $choose ?>').value = '<?php p(RESOURCE_LOCALPATH) ?>'+path;
         window.close();
     }
-    -->
+    //]]>
     </script>
     
     <br />
-    <div align="center" class="form">
-    <form name="myform">
-    <input type="file" size="60" name="myfile"><br />
+    <div class="form mdl-align">
+    <form id="myform">
+    <fieldset class="invisiblefieldset">
+    <input type="file" size="60" name="myfile" /><br />
     <input type="button" value="<?php print_string('localfileselect','resource') ?>" 
-           onClick="return set_value(document.myform.myfile.value)">
+           onClick="return set_value(getElementById('myform').myfile.value)">
     <input type="button" value="<?php print_string('cancel') ?>" 
            onClick="window.close()">
+    </fieldset>
     </form>
     </div>
 
-    </body>
-    </html>
+<?php
+    print_footer('empty');
+?>
