@@ -1,4 +1,4 @@
-<?php // $Id: graph.php,v 1.24.2.1 2005/10/29 16:40:54 skodak Exp $
+<?php // $Id: graph.php,v 1.30 2007/08/28 00:12:02 mattc-catalyst Exp $
 
     require_once("../../config.php");
     require_once("$CFG->libdir/graphlib.php");
@@ -10,7 +10,7 @@
     $sid   = optional_param('sid', false, PARAM_INT);  // Student ID
     $qid   = optional_param('qid', 0, PARAM_INT);  // Group ID
 
-    if (! $cm = get_record("course_modules", "id", $id)) {
+    if (! $cm = get_coursemodule_from_id('survey', $id)) {
         error("Course Module ID was incorrect");
     }
 
@@ -20,12 +20,13 @@
 
     require_login($course->id, false, $cm);
 
-    $groupmode = groupmode($course, $cm);   // Groups are being used
+    $groupmode = groups_get_activity_groupmode($cm);   // Groups are being used
+    $context = get_context_instance(CONTEXT_MODULE, $cm->id);
 
-    if (!isteacher($course->id)) {
+    if (!has_capability('mod/survey:readresponses', $context)) {
         if ($type != "student.png" or $sid != $USER->id ) {
             error("Sorry, you aren't allowed to see this.");
-        } else if ($groupmode and !ismember($group)) {
+        } else if ($groupmode and !groups_is_member($group)) {
             error("Sorry, you aren't allowed to see this.");
         }
     }
@@ -35,8 +36,10 @@
     }
 
 /// Check to see if groups are being used in this survey
-    if ($groupmode and $group) {
-        $users = get_group_users($group);
+    if ($group) {
+        $users = groups_get_members($group);
+    } else if (!empty($CFG->enablegroupings) && !empty($cm->groupingid)) { 
+        $users = groups_get_grouping_members($cm->groupingid);
     } else {
         $users = get_course_users($course->id);
         $group = false;
