@@ -1,4 +1,4 @@
-<?php // $Id: report.php,v 1.37 2006/04/27 21:30:18 skodak Exp $
+<?php // $Id: report.php,v 1.53.2.1 2007/10/12 16:09:38 tjhunt Exp $
 
     require_once("../../config.php");
     require_once("lib.php");
@@ -7,11 +7,15 @@
 
     $id      = required_param('id', PARAM_INT);           // Course Module ID
     $action  = optional_param('action', '', PARAM_ALPHA); // What to look at
-    $qid     = optional_param('qid', 0, PARAM_INT);       // Question ID
+    $qid     = optional_param('qid', 0, PARAM_RAW);       // Question IDs comma-separated list
     $student = optional_param('student', 0, PARAM_INT);   // Student ID
     $notes   = optional_param('notes', '', PARAM_RAW);    // Save teachers notes
 
-    if (! $cm = get_record("course_modules", "id", $id)) {
+    $qids = explode(',', $qid);
+    $qids = clean_param($qids, PARAM_INT);
+    $qid = implode (',', $qids);
+
+    if (! $cm = get_coursemodule_from_id('survey', $id)) {
         error("Course Module ID was incorrect");
     }
 
@@ -19,11 +23,11 @@
         error("Course is misconfigured");
     }
 
-    require_login($course->id, false);
+    require_login($course->id, false, $cm);
+    
+    $context = get_context_instance(CONTEXT_MODULE, $cm->id);
 
-    if (!isteacher($course->id)) {
-        error("Sorry, only teachers can see this.");
-    }
+    require_capability('mod/survey:readresponses', $context);
 
     if (! $survey = get_record("survey", "id", $cm->instance)) {
         error("Survey ID was incorrect");
@@ -52,7 +56,7 @@
 
     add_to_log($course->id, "survey", "view report", "report.php?id=$cm->id", "$survey->id", $cm->id);
 
-    if ($course->category) {
+    if ($course->id != SITEID) {
         $navigation = "<a href=\"../../course/view.php?id=$course->id\">$course->shortname</a> ->
                        <a href=\"index.php?id=$course->id\">$strsurveys</a> ->
                        <a href=\"view.php?id=$cm->id\">".format_string($survey->name,true)."</a> -> ";
@@ -60,40 +64,48 @@
         $navigation = "<a href=\"index.php?id=$course->id\">$strsurveys</a> ->
                        <a href=\"view.php?id=$cm->id\">".format_string($survey->name,true)."</a> -> ";
     }
-
-    print_header("$course->shortname: ".format_string($survey->name), "$course->fullname", "$navigation $strreport",
+    
+    $navigation = build_navigation($strreport, $cm);
+    print_header("$course->shortname: ".format_string($survey->name), $course->fullname, $navigation,
                  "", "", true,
                  update_module_button($cm->id, $course->id, $strsurvey), navmenu($course, $cm));
 
 /// Check to see if groups are being used in this survey
-    if ($groupmode = groupmode($course, $cm)) {   // Groups are being used
+    if ($groupmode = groups_get_activity_groupmode($cm)) {   // Groups are being used
         $menuaction = $action == "student" ? "students" : $action;
-        $currentgroup = setup_and_print_groups($course, $groupmode,
-                                       "report.php?id=$cm->id&amp;action=$menuaction&amp;qid=$qid");
+        $currentgroup = groups_get_activity_group($cm, true);
+        groups_print_activity_menu($cm, "report.php?id=$cm->id&amp;action=$menuaction&amp;qid=$qid");
     } else {
         $currentgroup = 0;
     }
 
     if ($currentgroup) {
-        $users = get_group_users($currentgroup);
+        $users = groups_get_members($currentgroup);
+    } else if (!empty($CFG->enablegroupings) && !empty($cm->groupingid)) { 
+        $users = groups_get_grouping_members($cm->groupingid);
     } else {
         $users = get_course_users($course->id);
     }
-
+    $groupingid = $cm->groupingid;
+    
     print_simple_box_start("center");
     if ($showscales) {
         echo "<a href=\"report.php?action=summary&amp;id=$id\">$strsummary</a>";
         echo "&nbsp;&nbsp;&nbsp;&nbsp;<a href=\"report.php?action=scales&amp;id=$id\">$strscales</a>";
         echo "&nbsp;&nbsp;&nbsp;&nbsp;<a href=\"report.php?action=questions&amp;id=$id\">$strquestions</a>";
         echo "&nbsp;&nbsp;&nbsp;&nbsp;<a href=\"report.php?action=students&amp;id=$id\">$course->students</a>";
-        echo "&nbsp;&nbsp;&nbsp;&nbsp;<a href=\"report.php?action=download&amp;id=$id\">$strdownload</a>";
+        if (has_capability('mod/survey:download', $context)) {
+            echo "&nbsp;&nbsp;&nbsp;&nbsp;<a href=\"report.php?action=download&amp;id=$id\">$strdownload</a>";
+        }
         if (empty($action)) {
             $action = "summary";
         }
     } else {
         echo "<a href=\"report.php?action=questions&amp;id=$id\">$strquestions</a>";
         echo "&nbsp;&nbsp;&nbsp;&nbsp;<a href=\"report.php?action=students&amp;id=$id\">$course->students</a>";
-        echo "&nbsp;&nbsp;&nbsp;&nbsp;<a href=\"report.php?action=download&amp;id=$id\">$strdownload</a>";
+        if (has_capability('mod/survey:download', $context)) {
+            echo "&nbsp;&nbsp;&nbsp;&nbsp;<a href=\"report.php?action=download&amp;id=$id\">$strdownload</a>";
+        }
         if (empty($action)) {
             $action = "questions";
         }
@@ -112,10 +124,10 @@
       case "summary":
         print_heading($strsummary);
 
-        if (survey_count_responses($survey->id, $currentgroup)) {
-            echo "<center><a href=\"report.php?action=scales&amp;id=$id\">";
+        if (survey_count_responses($survey->id, $currentgroup, $groupingid)) {
+            echo "<div class='reportsummary'><a href=\"report.php?action=scales&amp;id=$id\">";
             survey_print_graph("id=$id&amp;group=$currentgroup&amp;type=overall.png");
-            echo "</a></center>";
+            echo "</a></div>";
         } else {
             notify(get_string("nobodyyet","survey"));
         }
@@ -124,7 +136,7 @@
       case "scales":
         print_heading($strscales);
 
-        if (! $results = survey_get_responses($survey->id, $currentgroup) ) {
+        if (! $results = survey_get_responses($survey->id, $currentgroup, $groupingid) ) {
             notify(get_string("nobodyyet","survey"));
 
         } else {
@@ -146,7 +158,7 @@
                     if (!empty($virtualscales) && $question->type > 0) {  // Don't show non-virtual scales if virtual
                         continue;
                     }
-                    echo "<p align=\"center\"><a title=\"$strseemoredetail\" href=\"report.php?action=questions&amp;id=$id&amp;qid=$question->multi\">";
+                    echo "<p class=\"centerpara\"><a title=\"$strseemoredetail\" href=\"report.php?action=questions&amp;id=$id&amp;qid=$question->multi\">";
                     survey_print_graph("id=$id&amp;qid=$question->id&amp;group=$currentgroup&amp;type=multiquestion.png");
                     echo "</a></p><br />";
                 }
@@ -175,7 +187,7 @@
             print_heading($strallquestions);
         }
 
-        if (! $results = survey_get_responses($survey->id, $currentgroup) ) {
+        if (! $results = survey_get_responses($survey->id, $currentgroup, $groupingid) ) {
             notify(get_string("nobodyyet","survey"));
 
         } else {
@@ -204,14 +216,14 @@
                     foreach ($subquestionorder as $key => $val) {
                         $subquestion = $subquestions[$val];
                         if ($subquestion->type > 0) {
-                            echo "<p align=\"center\">";
+                            echo "<p class=\"centerpara\">";
                             echo "<a title=\"$strseemoredetail\" href=\"report.php?action=question&amp;id=$id&amp;qid=$subquestion->id\">";
                             survey_print_graph("id=$id&amp;qid=$subquestion->id&amp;group=$currentgroup&amp;type=question.png");
                             echo "</a></p>";
                         }
                     }
                 } else if ($question->type > 0 ) {
-                    echo "<p align=\"center\">";
+                    echo "<p class=\"centerpara\">";
                     echo "<a title=\"$strseemoredetail\" href=\"report.php?action=question&amp;id=$id&amp;qid=$question->id\">";
                     survey_print_graph("id=$id&amp;qid=$question->id&amp;group=$currentgroup&amp;type=question.png");
                     echo "</a></p>";
@@ -226,7 +238,7 @@
                     if ($aaa = survey_get_user_answers($survey->id, $question->id, $currentgroup, "sa.time ASC")) {
                         foreach ($aaa as $a) {
                             $contents .= "<tr>";
-                            $contents .= '<td nowrap="nowrap" width="10%" valign="top">'.fullname($a).'</td>';
+                            $contents .= '<td class="fullnamecell">'.fullname($a).'</td>';
                             $contents .= '<td valign="top">'.$a->answer1.'</td>';
                             $contents .= "</tr>";
                         }
@@ -295,7 +307,7 @@
 
          print_heading(get_string("analysisof", "survey", "$course->students"));
 
-         if (! $results = survey_get_responses($survey->id, $currentgroup) ) {
+         if (! $results = survey_get_responses($survey->id, $currentgroup, $groupingid) ) {
              notify(get_string("nobodyyet","survey"));
          } else {
              survey_print_all_responses($cm->id, $results, $course->id);
@@ -326,7 +338,7 @@
              }
          }
 
-         echo "<p align=\"center\">";
+         echo "<p <p class=\"centerpara\">";
          print_user_picture($user->id, $course->id, $user->picture, true);
          echo "</p>";
 
@@ -335,7 +347,7 @@
 
          if ($showscales) {
              // Print overall summary
-             echo "<p align=\"center\">";
+             echo "<p <p class=\"centerpara\">>";
              survey_print_graph("id=$id&amp;sid=$student&amp;type=student.png");
              echo "</p>";
 
@@ -355,7 +367,7 @@
                      if ($virtualscales && $question->type > 0) {  // Don't show non-virtual scales if virtual
                          continue;
                      }
-                     echo "<p align=\"center\">";
+                     echo "<p class=\"centerpara\">";
                      echo "<a title=\"$strseemoredetail\" href=\"report.php?action=questions&amp;id=$id&amp;qid=$question->multi\">";
                      survey_print_graph("id=$id&amp;qid=$question->id&amp;sid=$student&amp;type=studentmultiquestion.png");
                      echo "</a></p><br />";
@@ -385,8 +397,8 @@
             $notes = "";
          }
          echo "<hr noshade=\"noshade\" size=\"1\" />";
-         echo "<center>";
-         echo "<form action=\"report.php\" method=\"post\" name=\"form\">";
+         echo "<div class='studentreport'>";
+         echo "<form action=\"report.php\" method=\"post\">";
          echo "<h3>$strnotes:</h3>";
          echo "<blockquote>";
          echo "<textarea name=\"notes\" rows=\"10\" cols=\"60\">";
@@ -399,7 +411,7 @@
          echo "<input type=\"submit\" value=\"".get_string("savechanges")."\" />";
          echo "</blockquote>";
          echo "</form>";
-         echo "</center>";
+         echo "</div>";
 
 
          break;
@@ -407,17 +419,24 @@
       case "download":
         print_heading($strdownload);
 
-        echo '<p align="center">'.get_string("downloadinfo", "survey").'</p>';
+        require_capability('mod/survey:download', $context);
 
-        echo '<center>';
+        echo '<p class="centerpara">'.get_string("downloadinfo", "survey").'</p>';
+
+        echo '<div class="reportbuttons">';
+        $optons = array();
         $options["id"] = "$cm->id";
-        $options["type"] = "xls";
         $options["group"] = $currentgroup;
-        print_single_button("download.php", $options, get_string("downloadexcel", "survey"));
+
+        $options["type"] = "ods";
+        print_single_button("download.php", $options, get_string("downloadods"));
+
+        $options["type"] = "xls";
+        print_single_button("download.php", $options, get_string("downloadexcel"));
 
         $options["type"] = "txt";
-        print_single_button("download.php", $options, get_string("downloadtext", "survey"));
-        echo '</center>';
+        print_single_button("download.php", $options, get_string("downloadtext"));
+        echo '</div>';
 
         break;
 
