@@ -1,4 +1,4 @@
-<?php  // $Id: assessments.php,v 1.48 2006/04/19 20:21:42 skodak Exp $
+<?php  // $Id: assessments.php,v 1.62.2.1 2007/10/12 16:09:46 tjhunt Exp $
 
 /*************************************************
     ACTIONS handled are:
@@ -42,7 +42,7 @@
 
     // get some useful stuff...
     if ($id) {
-        if (! $cm = get_record("course_modules", "id", $id)) {
+        if (! $cm = get_coursemodule_from_id('workshop', $id)) {
             error("Course Module ID was incorrect");
         }
         if (! $workshop = get_record("workshop", "id", $cm->instance)) {
@@ -63,20 +63,15 @@
     }
 
     require_login($course->id, false, $cm);
-
-    $navigation = "";
-    if ($course->category) {
-        $navigation = "<a href=\"../../course/view.php?id=$course->id\">$course->shortname</a> ->";
-    }
+    $context = get_context_instance(CONTEXT_MODULE, $cm->id);
 
     $strworkshops = get_string("modulenameplural", "workshop");
     $strworkshop  = get_string("modulename", "workshop");
     $strassessments = get_string("assessments", "workshop");
 
     // ... print the header and...
-    print_header_simple(format_string($workshop->name), "",
-                 "<a href=\"index.php?id=$course->id\">$strworkshops</a> ->
-                  <a href=\"view.php?id=$cm->id\">".format_string($workshop->name,true)."</a> -> $strassessments",
+    $navigation = build_navigation($strassessments, $cm);
+    print_header_simple(format_string($workshop->name), "", $navigation,
                   "", "", true);
 
     /*************** add comment to assessment (by author, assessor or teacher) ***************************/
@@ -92,11 +87,11 @@
             error("Submission not found");
         }
         ?>
-        <form name="commentform" action="assessments.php" method="post">
+        <form id="commentform" action="assessments.php" method="post">
         <input type="hidden" name="action" value="insertcomment" />
         <input type="hidden" name="id" value="<?php echo $cm->id ?>" />
         <input type="hidden" name="aid" value="<?php echo $aid ?>" />
-        <center>
+        <div class="boxaligncenter">
         <table cellpadding="5" border="1">
         <?php
 
@@ -112,8 +107,8 @@
 
         echo "  </td></tr></table>\n";
         echo "<input type=\"submit\" value=\"".get_string("savemycomment", "workshop")."\" />\n";
-        echo "</center></form>\n";
-        echo "<center><b>".get_string("assessment", "workshop"). "</b></center>\n";
+        echo "</div></form>\n";
+        echo "<div style=\"text-align:center\"><b>".get_string("assessment", "workshop"). "</b></div>\n";
         workshop_print_assessment($workshop, $assessment);
     }
 
@@ -125,9 +120,7 @@
             error("Workshop Assessment ID and/or Element Number missing");
         }
 
-        if (!isteacher($course->id)) {
-            error("Only teachers can look at this page");
-        }
+        require_capability('mod/workshop:manage', $context);
 
         if (!$assessment = get_record("workshop_assessments", "id", $aid)) {
             error("workshop assessment is misconfigured");
@@ -352,9 +345,7 @@
     /*********************** admin list of asssessments (of a submission) (by teachers)**************/
     elseif ($action == 'adminlist') {
 
-        if (!isteacher($course->id)) {
-            error("Only teachers can look at this page");
-            }
+        require_capability('mod/workshop:manage', $context);
 
         if (empty($sid)) {
             error ("Workshop asssessments: adminlist called with no sid");
@@ -368,9 +359,7 @@
     /*********************** admin list of asssessments by a student (used by teachers only )******************/
     elseif ($action == 'adminlistbystudent') {
 
-        if (!isteacher($course->id)) {
-            error("Only teachers can look at this page");
-            }
+        require_capability('mod/workshop:manage', $context);
 
         if (empty($userid)) {
             error ("Workshop asssessments: adminlistbystudent called with no userid");
@@ -390,7 +379,7 @@
             }
         //save time of agreement
         set_field("workshop_assessments", "timeagreed", $timenow, "id", $assessment->id);
-        echo "<centre><b>".get_string("savedok", "workshop")."</b></center><br />\n";
+        echo "<div style=\"text-align:center\"><b>".get_string("savedok", "workshop")."</b></div><br />\n";
 
         add_to_log($course->id, "workshop", "agree", "viewassessment.php?id=$cm->id&amp;aid=$assessment->id", "$assessment->id");
         print_continue("view.php?id=$cm->id");
@@ -423,11 +412,11 @@
             error("Edit Comment: Submission not found");
             }
         ?>
-        <form name="gradingform" action="assessments.php" method="post">
+        <form id="gradingform" action="assessments.php" method="post">
         <input type="hidden" name="action" value="updatecomment" />
         <input type="hidden" name="id" value="<?php echo $cm->id ?>" />
         <input type="hidden" name="cid" value="<?php echo $cid ?>" />
-        <center>
+        <div class="boxaligncenter">
         <table cellpadding="5" border="1">
         <?php
 
@@ -442,7 +431,7 @@
         echo "      </textarea>\n";
         echo "  </td></tr></table>\n";
         echo "<input type=\"submit\" value=\"".get_string("savemycomment", "workshop")."\" />\n";
-        echo "</center></form>\n";
+        echo "</div></form>\n";
         workshop_print_assessment($workshop, $assessment);
         }
 
@@ -450,9 +439,7 @@
     /*********************** edit assessment elements (for teachers) ***********************/
     elseif ($action == 'editelements') {
 
-        if (!isteacher($course->id)) {
-            error("Only teachers can look at this page");
-        }
+        require_capability('mod/workshop:manage', $context);
 
         $count = count_records("workshop_grades", "workshopid", $workshop->id);
         if ($count) {
@@ -461,10 +448,10 @@
         // set up heading, form and table
         print_heading_with_help(get_string("editingassessmentelements", "workshop"), "elements", "workshop");
         ?>
-        <form name="form" method="post" action="assessments.php">
+        <form id="form" method="post" action="assessments.php">
         <input type="hidden" name="id" value="<?php echo $cm->id ?>" />
         <input type="hidden" name="action" value="insertelements" />
-        <center><table cellpadding="5" border="1">
+        <div class="boxaligncenter"><table cellpadding="5" border="1">
         <?php
 
         // get existing elements, if none set up appropriate default ones
@@ -545,9 +532,9 @@
                     echo "  <td colspan=\"2\" class=\"workshopassessmentheading\">&nbsp;</td>\n";
                     echo "</tr>\n";
                 }
-                echo "</center></table><br />\n";
-                echo "<center><b>".get_string("gradetable","workshop")."</b></center>\n";
-                echo "<center><table cellpadding=\"5\" border=\"1\"><tr><td align=\"CENTER\">".
+                echo "</div></table><br />\n";
+                echo "<div style=\"text-align:center\"><b>".get_string("gradetable","workshop")."</b></div>\n";
+                echo "<div class=\"boxaligncenter\"><table cellpadding=\"5\" border=\"1\"><tr><td align=\"CENTER\">".
                     get_string("numberofnegativeresponses", "workshop");
                 echo "</td><td>". get_string("suggestedgrade", "workshop")."</td></tr>\n";
                 for ($j = $workshop->grade; $j >= 0; $j--) {
@@ -562,7 +549,7 @@
                     choose_from_menu($numbers, "maxscore[$i]", $elements[$i]->maxscore, "");
                     echo "</td></tr>\n";
                 }
-                echo "</table></center>\n";
+                echo "</table></div>\n";
                 break;
 
             case 3: // criterion grading
@@ -626,7 +613,7 @@
         </table><br />
         <input type="submit" value="<?php  print_string("savechanges") ?>" />
         <input type="submit" name="cancel" value="<?php  print_string("cancel") ?>" />
-        </center>
+        </div>
         </form>
         <?php
     }
@@ -635,9 +622,7 @@
     /*************** grade all assessments (by teacher) ***************************/
     elseif ($action == 'gradeallassessments') {
 
-        if (!isteacher($course->id)) {
-            error("Only teachers can look at this page");
-        }
+        require_capability('mod/workshop:manage', $context);
 
         print_heading(get_string("gradingallassessments", "workshop"));
         workshop_grade_assessments($workshop);
@@ -648,9 +633,7 @@
     /*************** grade (student's) assessment (by teacher) ***************************/
     elseif ($action == 'gradeassessment') {
 
-        if (!isteacher($course->id)) {
-            error("Only teachers can look at this page");
-        }
+        require_capability('mod/workshop:manage', $context);
 
         print_heading_with_help(get_string("gradeassessment", "workshop"), "gradingassessments", "workshop");
         // get assessment record
@@ -663,12 +646,12 @@
         }
         // get the teacher's assessment first
         if ($teachersassessment = workshop_get_submission_assessment($submission, $USER)) {
-            echo "<center><b>".get_string("teacherassessments", "workshop", $course->teacher)."</b></center>\n";
+            echo "<div style=\"text-align:center\"><b>".get_string("teacherassessments", "workshop", $course->teacher)."</b></div>\n";
             workshop_print_assessment($workshop, $teachersassessment);
         }
         // now the student's assessment (don't allow changes)
         $user = get_record("user", "id", $assessment->userid);
-        echo "<center><b>".get_string("assessmentby", "workshop", $user->firstname." ".$user->lastname)."</b></center>\n";
+        echo "<div style=\"text-align:center\"><b>".get_string("assessmentby", "workshop", $user->firstname." ".$user->lastname)."</b></div>\n";
         workshop_print_assessment($workshop, $assessment);
 
         include('assessment_grading_form.html');
@@ -704,9 +687,7 @@
     /*********************** insert/update assignment elements (for teachers)***********************/
     elseif ($action == 'insertelements') {
 
-        if (!isteacher($course->id)) {
-            error("Only teachers can look at this page");
-        }
+        require_capability('mod/workshop:manage', $context);
 
         $form = data_submitted();
 
@@ -813,7 +794,7 @@
                 }
                 break;
         } // end of switch
-
+        echo "</div>"; // not sure where this one came from MDL-7861
         redirect("view.php?id=$cm->id", get_string("savedok","workshop"));
     }
 
@@ -821,9 +802,8 @@
     /*********************** list assessments for grading (Student submissions)(by teachers)***********************/
     elseif ($action == 'listungradedstudentsubmissions') {
 
-        if (!isteacher($course->id)) {
-            error("Only teachers can look at this page");
-            }
+        require_capability('mod/workshop:manage', $context);
+
         workshop_list_ungraded_assessments($workshop, "student");
         print_continue("view.php?id=$cm->id");
         }
@@ -832,9 +812,8 @@
     /*********************** list assessments for grading (Teacher submissions) (by teachers)***********************/
     elseif ($action == 'listungradedteachersubmissions') {
 
-        if (!isteacher($course->id)) {
-            error("Only teachers can look at this page");
-            }
+        require_capability('mod/workshop:manage', $context);
+
         workshop_list_ungraded_assessments($workshop, "teacher");
         print_continue("view.php?id=$cm->id");
         }
@@ -852,9 +831,7 @@
     elseif ($action == 'regradestudentassessments' ) {
 
         $timenow = time();
-        if (!isteacher($course->id)) {
-            error("Only teachers can look at this page");
-        }
+        require_capability('mod/workshop:manage', $context);
         // get all the submissions...
         if ($submissions = get_records("workshop_submissions", "workshopid", $workshop->id)) {
             foreach ($submissions as $submission) {
@@ -879,9 +856,7 @@
             error("Workshop Assessment id and/or Stock Comment id missing");
         }
 
-        if (!isteacher($course->id)) {
-            error("Only teachers can look at this page");
-        }
+        require_capability('mod/workshop:manage', $context);
 
         if (!$assessment = get_record("workshop_assessments", "id", $aid)) {
             error("workshop assessment is misconfigured");
@@ -1250,12 +1225,12 @@
             workshop_grade_assessments($workshop);
         } else { // it could be self assessment....
             // now see if there's a corresponding assessment so that the gradinggrade can be set
-            if (isteacher($course->id)) {
+            if (workshop_is_teacher($workshop)) {
                 // see if there's are student assessments, if so set their gradinggrade
                 if ($assessments = workshop_get_assessments($submission)) {
                     foreach($assessments as $studentassessment) {
                         // skip if it's not a student assessment
-                        if (!isstudent($course->id, $studentassessment->userid)) {
+                        if (!workshop_is_student($workshop, $studentassessment->userid)) {
                             continue;
                         }
                         $gradinggrade = workshop_compare_assessments($workshop, $assessment, $studentassessment);
@@ -1266,7 +1241,7 @@
             } else { //it's a student assessment, see if there's a corresponding teacher's assessment
                 if ($assessments = workshop_get_assessments($submission)) {
                     foreach($assessments as $teacherassessment) {
-                        if (isteacher($course->id, $teacherassessment->userid)) {
+                        if (workshop_is_teacher($workshop, $teacherassessment->userid)) {
                             $gradinggrade = workshop_compare_assessments($workshop, $assessment, $teacherassessment);
                             set_field("workshop_assessments", "timegraded", $timenow, "id", $assessment->id);
                             set_field("workshop_assessments", "gradinggrade", $gradinggrade, "id", $assessment->id);
@@ -1292,6 +1267,7 @@
 
         // show grade if grading strategy is not zero
         if ($workshop->gradingstrategy) {
+            echo "</div>"; // MDL-7861, this is from <div id=page>
             redirect($returnto, get_string("thegradeis", "workshop").": ".
                     number_format($grade * $workshop->grade / 100, 2).
                     " (".get_string("maximumgrade")." ".number_format($workshop->grade).")");
@@ -1299,6 +1275,7 @@
         else {
             redirect($returnto);
         }
+        
     }
 
 
@@ -1321,7 +1298,7 @@
             set_field("workshop_comments", "timecreated", $timenow, "id", $comment->id);
             // ..and kick to comment into life (probably not needed but just in case)
             set_field("workshop_comments", "mailed", 0, "id", $comment->id);
-            echo "<centre><b>".get_string("savedok", "workshop")."</b></center><br />\n";
+            echo "<centre><b>".get_string("savedok", "workshop")."</b></div><br />\n";
 
             add_to_log($course->id, "workshop", "comment",
                     "viewassessment.php?id=$cm->id&amp;aid=$assessment->id", "$comment->id");
@@ -1335,9 +1312,7 @@
     elseif ($action == 'updategrading') {
         $timenow = time();
 
-        if (!isteacher($course->id)) {
-            error("Only teachers can look at this page");
-        }
+        require_capability('mod/workshop:manage', $context);
 
         $form = (object)$_POST;
 
@@ -1380,8 +1355,6 @@
     else {
         error("Fatal Error: Unknown Action: ".$action."\n");
     }
-
     print_footer($course);
 
 ?>
-
