@@ -1,4 +1,4 @@
-<?php  // $Id: upload.php,v 1.25 2006/04/19 19:48:47 skodak Exp $
+<?php  // $Id: upload.php,v 1.32.2.2 2009/11/21 23:10:44 skodak Exp $
 
     require("../../config.php");
     require("lib.php");
@@ -7,7 +7,7 @@
     $id = required_param('id', PARAM_INT);          // CM ID
 
 
-    if (! $cm = get_record("course_modules", "id", $id)) {
+    if (! $cm = get_coursemodule_from_id('workshop', $id)) {
         error("Course Module ID was incorrect");
     }
     if (! $course = get_record("course", "id", $cm->course)) {
@@ -23,9 +23,8 @@
     $strworkshop = get_string('modulename', 'workshop');
     $strsubmission = get_string('submission', 'workshop');
 
-    print_header_simple(format_string($workshop->name)." : $strsubmission", "",
-                 "<a href=\"index.php?id=$course->id\">$strworkshops</a> ->
-                  <a href=\"view.php?a=$workshop->id\">".format_string($workshop->name,true)."</a> -> $strsubmission",
+    $navigation = build_navigation($strsubmission, $cm);
+    print_header_simple(format_string($workshop->name)." : $strsubmission", "", $navigation,
                   "", "", true);
     $timenow = time();
 
@@ -38,7 +37,7 @@
 
     // check that this is not a "rapid" second submission, caused by using the back button
     // only check if a student, teachers may want to submit a set of workshop examples rapidly
-    if (isstudent($course->id)) {
+    if (workshop_is_student($workshop)) {
         if ($submissions = workshop_get_user_submissions($workshop, $USER)) {
             // returns all submissions, newest on first
             foreach ($submissions as $submission) {
@@ -117,4 +116,3 @@
     print_footer($course);
 
 ?>
-
